"""These DATABASE commands are used to select subsets of database
entities for further operations.
"""

from typing import Optional, Union
from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat


def allsel(self, labt="", entity="", **kwargs):
    """Selects all entities with a single command.

    APDL Command: ALLSEL

    Parameters
    ----------
    labt
        Type of selection to be made:

        ALL - Selects all items of the specified entity type and all items of lower entity
              types (default).

        BELOW - Selects all items directly associated with and below the selected items of the
                specified entity type.

    entity
        Entity type on which selection is based:

        ALL - All entity types (default).

        VOLU - Volumes.

        AREA - Areas.

        LINE - Lines.

        KP - Keypoints.

        ELEM - Elements.

        NODE - Nodes.

    Notes
    -----
    ALLSEL is a convenience command that allows the user to select all
    items of a specified entity type or to select items associated with the
    selected items of a higher entity.

    An entity hierarchy is used to decide what entities will be available
    in the selection process.  This hierarchy from top to bottom is as
    follows:  volumes, areas, lines, keypoints, elements, and nodes.  The
    hierarchy may also be divided into two branches:  the solid model and
    the finite element model.  The label ALL selects items based on one
    branch only, while BELOW uses the entire entity hierarchy.  For
    example, ALLSEL,ALL,VOLU selects all volumes, areas, lines, and
    keypoints in the data base.  ALLSEL,BELOW,AREA selects all lines
    belonging to the selected areas; all keypoints belonging to those
    lines; all elements belonging to those areas, lines, and keypoints; and
    all nodes belonging to those elements.

    The $ character should not be used after the  ALLSEL  command.

    This command is valid in any processor.
    """
    command = "ALLSEL,%s,%s" % (str(labt), str(entity))
    return self.run(command, **kwargs)


def asll(self, type_="", arkey="", **kwargs):
    """Selects those areas containing the selected lines.

    APDL Command: ASLL

    Parameters
    ----------
    type\_
        Label identifying the type of area select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    arkey
        Specifies whether all contained area lines must be selected [LSEL]:

        0 - Select area if any of its lines are in the selected line set.

        1 - Select area only if all of its lines are in the selected line set.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "ASLL,%s,%s" % (str(type_), str(arkey))
    return self.run(command, **kwargs)


def asel(self, type_="", item="", comp="", vmin="", vmax="", vinc="",
         kswp="", **kwargs):
    """Selects a subset of areas.

    APDL Command: ASEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        S - Select a new set (default)

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

        ALL - Restore the full set.

        NONE - Unselect the full set.

        INVE - Invert the current set (selected becomes unselected and vice versa).

        STAT - Display the current select status.

    Notes
    -----
    Selects a subset of areas. For example, to select those areas with area
    numbers 1 through 7, use ASEL,S,AREA,,1,7.  The selected subset is then
    used when the ALL label is entered (or implied) on other commands, such
    as ALIST,ALL.  Only data identified by area number are selected.  Data
    are flagged as selected and unselected; no data are actually deleted
    from the database.

    In a cyclic symmetry analysis, area hot spots can be modified.
    Consequently, the result of an area selection may be different before
    and after the CYCLIC command.

    If Item = ACCA, the command selects only those areas that were created
    by concatenation.  The KSWP field is processed, but the Comp, VMIN,
    VMAX, and VINC fields are ignored.

    This command is valid in any processor.

    For Selects based on non-integer numbers (coordinates, results, etc.),
    items that are within the range VMIN-Toler and VMAX+Toler are selected.
    The default tolerance Toler is based on the relative values of VMIN and
    VMAX as follows:

    If VMIN = VMAX, Toler = 0.005 x VMIN.

    If VMIN = VMAX = 0.0, Toler = 1.0E-6.

     If VMAX ≠ VMIN, Toler = 1.0E-8 x (VMAX-VMIN).

    Use the SELTOL command to override this default and specify Toler
    explicitly.

    Table: 127:: : ASEL - Valid Item and Component Labels
    """
    command = "ASEL,%s,%s,%s,%s,%s,%s,%s" % (str(type_), str(
        item), str(comp), str(vmin), str(vmax), str(vinc), str(kswp))
    return self.run(command, **kwargs)


def aslv(self, type_="", **kwargs):
    """Selects those areas contained in the selected volumes.

    APDL Command: ASLV

    Parameters
    ----------
    type\_
        Label identifying the type of area select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "ASLV,%s" % (str(type_))
    return self.run(command, **kwargs)


def dofsel(self, type_="", dof1="", dof2="", dof3="", dof4="", dof5="",
           dof6="", **kwargs):
    """Selects a DOF label set for reference by other commands.

    APDL Command: DOFSEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        S - Select a new set of labels.

        A - Add labels to the current set.

        U - Unselect (remove) labels from the current set.

        ALL - Restore the full set of labels.

        STAT - Display the current select status.

    dof1, dof2, dof3, . . . , dof6
        Used only with Type = S, A, or U.  Valid structural labels:  UX,
        UY, or UZ (displacements); U (UX, UY, and UZ);  ROTX, ROTY, or ROTZ
        (rotations); ROT (ROTX, ROTY, and ROTZ);  DISP (U and ROT); HDSP
        (Hydrostatic pressure). Valid thermal labels: TEMP, TBOT, TE2, TE3,
        . . ., TTOP (temperature).  Valid acoustic labels:  PRES
        (pressure); UX, UY, or UZ (displacements for FSI coupled elements).
        Valid electric labels:  VOLT (voltage); EMF (electromotive force
        drop); CURR (current).  Valid magnetic labels:  MAG (scalar
        magnetic potential); AX, AY or AZ (vector magnetic potentials); A
        (AX, AY and AZ); CURR (current).  Valid structural force labels:
        FX, FY, or FZ (forces); F (FX, FY, and FZ); MX, MY, or MZ
        (moments); M (MX, MY, and MZ);  FORC (F and M); DVOL (fluid mass
        flow rate).  Valid thermal force labels:  HEAT, HBOT, HE2, HE3, . .
        ., HTOP (heat flow).  Valid fluid flow force labels:  FLOW (fluid
        flow).  Valid electric force labels:  AMPS (current flow); CHRG
        (electric charge).  Valid magnetic force labels:  FLUX (scalar
        magnetic flux); CSGX, CSGY, or CSGZ (magnetic current segments);
        CSG (CSGX, CSGY, and CSGZ). Valid diffusion labels: CONC
        (concentration); RATE (diffusion flow rate).

    Notes
    -----
    Selects a degree of freedom label set for reference by other commands.
    The label set is used on certain commands where ALL is either input in
    the degree of freedom label field or implied.  The active label set has
    no effect on the solution degrees of freedom.  Specified labels which
    are not active in the model (from the ET or DOF command) are ignored.
    As a convenience, a set of force labels corresponding to the degree of
    freedom labels is also selected.  For example, selecting UX also causes
    FX to be selected (and vice versa).  The force label set is used on
    certain commands where ALL is input in the force label field.

    This command is valid in any processor.
    """
    command = "DOFSEL,%s,%s,%s,%s,%s,%s,%s" % (str(type_), str(
        dof1), str(dof2), str(dof3), str(dof4), str(dof5), str(dof6))
    return self.run(command, **kwargs)


def esel(self, type_: str = "", item: str = "", comp: str = "",
         vmin: Union[str, int, float] = "",
         vmax: Union[str, int, float] = "", vinc: MapdlInt = "",
         kabs: MapdlInt = "", **kwargs) -> Optional[str]:
    """Selects a subset of elements.

    APDL Command: ESEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        - S - Select a new set (default).
        - R - Reselect a set from the current set.
        - A - Additionally select a set and extend the current set.
        - U - Unselect a set from the current set.
        - ALL - Restore the full set.
        - NONE - Unselect the full set.
        - INVE - Invert the current set (selected becomes
          unselected and vice versa).
        - STAT - Display the current select status.

    item
        Label identifying data, see Table 110: ESEL - Valid Item
        and Component Labels. Some items also require a
        component label. Defaults to ELEM. If Item = STRA
        (straightened), elements are selected whose midside nodes
        do not conform to the curved line or non-flat area on
        which they should lie. (Such elements are sometimes
        formed during volume meshing (VMESH) in an attempt to
        avoid excessive element distortion.) You should
        graphically examine any such elements to evaluate their
        possible effect on solution accuracy.

    comp
        Component of the item (if required). Valid component
        labels are shown in Table 110: ESEL - Valid Item and
        Component Labels below.

    vmin
        Minimum value of item range. Ranges are element numbers,
        attribute numbers, load values, or result values
        as appropriate for the item. A component name (as
        specified via the CM command) can also be substituted for
        VMIN (in which case VMAX and VINC are ignored).

    vmax
        Maximum value of item range. VMAX defaults to VMIN for
        input values. For result values, VMAX defaults to infinity if VMIN is
        positive, or to zero if VMIN is negative.

    vinc
        Value increment within range. Used only with integer
        ranges (such as for element and attribute numbers).
        Defaults to 1. VINC cannot be negative.

    kabs
        Absolute value key:

            - `kabs = 0` - Check sign of value during selection.
            - `kabs = 1` - Use absolute value during selection (sign
              ignored).

    Notes
    -----
    The fields `item`, `comp`, `vmin`, `vmax`, `vinc` and `kabs` are
    used only with `type_` = `"S"`, `"R"`, `"A"`, or `"U"`.

    Selects elements based on values of a labeled item and
    component. For example, to select a new set of elements
    based on element numbers 1
    through 7, use ESEL,S,ELEM,,1,7.  The subset is used when the
    ALL label is entered (or implied) on other commands, such as
    ELIST, ALL. Only data identified by element number are
    selected. Selected data are internally flagged; no actual
    removal of data from the database occurs. Different element
    subsets cannot be used for different load steps [SOLVE] in a
    /SOLU sequence.  The subset used in the first load step
    will be used for all subsequent load steps regardless of
    subsequent ESEL specifications.

    This command is valid in any processor.

    Elements crossing the named path (see PATH command) will be
    selected. This option is only available in PREP7 and POST1.
    If no geometry data has been mapped to the path (i.e.,
    via PMAP and PDEF commands), the path will assume the default
    mapping option (PMAP,UNIFORM) to map the geometry prior to
    selecting the elements. If an invalid path name is
    given, the ESEL command is ignored (status of selected
    elements is unchanged). If there are no elements crossing the
    path, the ESEL command will return zero elements selected.

    For selections based on non-integer numbers (coordinates,
    results, etc.), items that are within the range VMIN -Toler
    and VMAX + Toler are selected. The default tolerance Toler is
    based on the relative values of VMIN and VMAX as follows:

    If VMIN = VMAX, Toler = 0.005 x VMIN.

    If VMIN = VMAX = 0.0, Toler = 1.0E-6.

    If VMAX ≠ VMIN, Toler = 1.0E-8 x (VMAX - VMIN).

    Use the SELTOL command to override this default and specify
    Toler explicitly.

    Table: 133:: : ESEL - Valid Item and Component Labels
    """
    command = f"ESEL,{type_},{item},{comp},{vmin},{vmax},{vinc}," \
              f"{kabs}"
    return self.run(command, **kwargs)


def esla(self, type_: str = "", **kwargs) -> Optional[str]:
    """Selects those elements associated with the selected areas.

    APDL Command: ESLA

    Parameters
    ----------
    type_
        Label identifying the type of element select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    Selects area elements belonging to meshed [AMESH], selected
    [ASEL] areas.

    This command is valid in any processor.
    """
    command = f"ESLA,{type_}"
    return self.run(command, **kwargs)


def esll(self, type_: str = "", **kwargs) -> Optional[str]:
    """Selects those elements associated with the selected lines.

    APDL Command: ESLL

    Parameters
    ----------
    type_
        Label identifying the type of element select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    Selects line elements belonging to meshed [LMESH], selected
    [LSEL] lines.

    This command is valid in any processor.
    """
    command = f"ESLL,{type_}"
    return self.run(command, **kwargs)


def esln(self, type_: str = "", ekey: MapdlInt = "",
         nodetype: str = "", **kwargs) -> Optional[str]:
    """Selects those elements attached to the selected nodes.

    APDL Command: ESLN

    Parameters
    ----------
    type_
        Label identifying the type of element selected:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    ekey
        Node set key:

        0 - Select element if any of its nodes are in the
        selected nodal set (default).

        1 - Select element only if all of its nodes are in the
        selected nodal set.

    nodetype
        Label identifying type of nodes to consider when selecting:

        ALL - Select elements considering all of their nodes (
              default).

        ACTIVE - Select elements considering only their active
                 nodes. An active node is a node
                 that contributes DOFs to the model.

        INACTIVE - Select elements considering only their
                   inactive nodes (such as orientation or
                   radiation nodes).

        CORNER - Select elements considering only their corner
                 nodes.

        MID - Select elements considering only their midside nodes.

    Notes
    -----
    ESLN selects elements which have any (or all EKEY) NodeType
    nodes in the currently-selected set of nodes. Only elements
    having nodes in the currently-selected set can be selected.

    This command is valid in any processor.
    """
    command = f"ESLN,{type_},{ekey},{nodetype}"
    return self.run(command, **kwargs)


def eslv(self, type_: str = "", **kwargs) -> Optional[str]:
    """Selects elements associated with the selected volumes.

    APDL Command: ESLV

    Parameters
    ----------
    type_
        Label identifying the type of element selected:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    Selects volume elements belonging to meshed [VMESH], selected
    [VSEL]
    volumes.

    This command is valid in any processor.
    """
    command = f"ESLV,{type_}"
    return self.run(command, **kwargs)


def ksel(self, type_="", item="", comp="", vmin="", vmax="", vinc="",
         kabs="", **kwargs):
    """Selects a subset of keypoints or hard points.

    APDL Command: KSEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

        ALL - Restore the full set.

        NONE - Unselect the full set.

        INVE - Invert the current set (selected becomes unselected and vice versa).

        STAT - Display the current select status.

    Notes
    -----
    Selects a subset of keypoints or hard points.  For example, to select a
    new set of keypoints based on keypoint numbers 1 through 7, use
    KSEL,S,KP,,1,7.  The selected subset is used when the ALL label is
    entered (or implied) on other commands, such as KLIST,ALL.  Only data
    identified by keypoint number are selected.  Data are flagged as
    selected and unselected; no data are actually deleted from the
    database.

    This command is valid in any processor.

    For selections based on non-integer numbers (coordinates, results,
    etc.), items that are within the range VMIN -Toler and VMAX + Toler are
    selected. The default tolerance Toler is based on the relative values
    of VMIN and VMAX as follows:

    If VMIN = VMAX, Toler = 0.005 x VMIN.

    If VMIN = VMAX = 0.0, Toler = 1.0E-6.

     If VMAX ≠ VMIN, Toler = 1.0E-8 x (VMAX - VMIN).

    Use the SELTOL command to override this default and specify Toler
    explicitly.

    Table: 203:: : KSEL - Valid Item and Component Labels
    """
    command = "KSEL,%s,%s,%s,%s,%s,%s,%s" % (str(type_), str(
        item), str(comp), str(vmin), str(vmax), str(vinc), str(kabs))
    return self.run(command, **kwargs)


def ksll(self, type_="", **kwargs):
    """Selects those keypoints contained in the selected lines.

    APDL Command: KSLL

    Parameters
    ----------
    type\_
        Label identifying the type of keypoint select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "KSLL,%s" % (str(type_))
    return self.run(command, **kwargs)


def ksln(self, type_="", **kwargs):
    """Selects those keypoints associated with the selected nodes.

    APDL Command: KSLN

    Parameters
    ----------
    type\_
        Label identifying the type of keypoint select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    Valid only if the nodes were generated by a meshing operation [KMESH,
    LMESH, AMESH, VMESH] on a solid model that contains the associated
    keypoints.

    This command is valid in any processor.
    """
    command = "KSLN,%s" % (str(type_))
    return self.run(command, **kwargs)


def lsel(self, type_="", item="", comp="", vmin="", vmax="", vinc="",
         kswp="", **kwargs):
    """Selects a subset of lines.

    APDL Command: LSEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

        ALL - Restore the full set.

        NONE - Unselect the full set.

        INVE - Invert the current set (selected becomes unselected and vice versa).

        STAT - Display the current select status.

    Notes
    -----
    Selects lines based on values of a labeled item and component.  For
    example, to select a new set of lines based on line numbers 1 through
    7, use LSEL,S,LINE,,1,7.  The subset is used when the ALL label is
    entered (or implied) on other commands, such as LLIST,ALL.  Only data
    identified by line number are selected.  Data are flagged as selected
    and unselected; no data are actually deleted from the database.

    If Item = LCCA, the command selects only those lines that were created
    by concatenation.  The KSWP field is processed, but the Comp, VMIN,
    VMAX, and VINC fields are ignored.

    If Item = HPT, the command selects only those lines that contain hard
    points.

    Item = RADIUS is only valid for lines that are circular arcs.

    LSEL is valid in any processor.

    For selections based on non-integer numbers (coordinates, results,
    etc.), items that are within the range VMIN -Toler and VMAX +Toler are
    selected. The default tolerance Toler is based on the relative values
    of VMIN and VMAX as follows:

    If VMIN = VMAX, Toler = 0.005 x VMIN.

    If VMIN = VMAX = 0.0, Toler = 1.0E-6.

    If VMAX ≠ VMIN, Toler = 1.0E-8 x (VMAX - VMIN).

    Use the SELTOL command to override this default and specify Toler
    explicitly.

    Table: 204:: : LSEL - Valid Item and Component Labels
    """
    command = "LSEL,%s,%s,%s,%s,%s,%s,%s" % (str(type_), str(
        item), str(comp), str(vmin), str(vmax), str(vinc), str(kswp))
    return self.run(command, **kwargs)


def lsla(self, type_="", **kwargs):
    """Selects those lines contained in the selected areas.

    APDL Command: LSLA

    Parameters
    ----------
    type\_
        Label identifying the type of line select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "LSLA,%s" % (str(type_))
    return self.run(command, **kwargs)


def lslk(self, type_="", lskey="", **kwargs):
    """Selects those lines containing the selected keypoints.

    APDL Command: LSLK

    Parameters
    ----------
    type\_
        Label identifying the type of line select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    lskey
        Specifies whether all contained line keypoints must be selected
        [KSEL]:

        0 - Select line if any of its keypoints are in the selected keypoint set.

        1 - Select line only if all of its keypoints are in the selected keypoint set.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "LSLK,%s,%s" % (str(type_), str(lskey))
    return self.run(command, **kwargs)


def nsel(self, type_="", item="", comp="", vmin="", vmax="", vinc="",
         kabs="", **kwargs):
    """Selects a subset of nodes.

    APDL Command: NSEL

    Parameters
    ----------
    type\_
        Label identifying the type of select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

        ALL - Restore the full set.

        NONE - Unselect the full set.

        INVE - Invert the current set (selected becomes unselected and vice versa).

        STAT - Display the current select status.

    Notes
    -----
    Selects a subset of nodes.  For example, to select a new set of nodes
    based on node numbers 1 through 7, use NSEL,S,NODE,,1,7.  The subset is
    used when the ALL label is entered (or implied) on other commands, such
    as NLIST,ALL.  Only data identified by node number are selected.  Data
    are flagged as selected and unselected; no data are actually deleted
    from the database.

    When selecting nodes by results, the full graphics value is used,
    regardless of whether PowerGraphics is on.

    Solution result data consists of two types, 1) nodal degree of freedom
    --results initially calculated at the nodes (such as displacement,
    temperature, pressure, etc.), and 2) element--results initially
    calculated elsewhere (such as at an element integration point or
    thickness location) and then recalculated at the nodes (such as
    stresses, strains, etc.).  Various element results also depend upon the
    recalculation method and the selected results location [AVPRIN, RSYS,
    FORCE, LAYER and SHELL].

    You must have all the nodes (corner and midside nodes) on the external
    face of the element selected to use Item = EXT.

    This command is valid in any processor.

    For Selects based on non-integer numbers (coordinates, results, etc.),
    items that are within the range VMIN-Toler and VMAX+Toler are selected.
    The default tolerance Toler is based on the relative values of VMIN and
    VMAX as follows:

    If VMIN = VMAX, Toler = 0.005 x VMIN.

    If VMIN = VMAX = 0.0, Toler = 1.0E-6.

     If VMAX ≠ VMIN, Toler = 1.0E-8 x (VMAX-VMIN).

    Use the SELTOL command to override this default and specify Toler
    explicitly.

    Table: 208:: : NSEL - Valid Item and Component Labels

    Table: 209:: : NSEL - Valid Item and Component Labels for Nodal DOF
    Result Values
    """
    command = "NSEL,%s,%s,%s,%s,%s,%s,%s" % (str(type_), str(
        item), str(comp), str(vmin), str(vmax), str(vinc), str(kabs))
    return self.run(command, **kwargs)


def nsla(self, type_="", nkey="", **kwargs):
    """Selects those nodes associated with the selected areas.

    APDL Command: NSLA

    Parameters
    ----------
    type\_
        Label identifying the type of node select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    nkey
        Specifies whether only interior area nodes are to be selected:

        0 - Select only nodes interior to selected areas.

        1 - Select all nodes (interior to area, interior to lines, and at keypoints)
            associated with the selected areas.

    Notes
    -----
    Valid only if the nodes were generated by an area meshing operation
    [AMESH, VMESH] on a solid model that contains the selected areas.

    This command is valid in any processor.
    """
    command = "NSLA,%s,%s" % (str(type_), str(nkey))
    return self.run(command, **kwargs)


def nsle(self, type_="", nodetype="", num="", **kwargs):
    """Selects those nodes attached to the selected elements.

    APDL Command: NSLE

    Parameters
    ----------
    type\_
        Label identifying the type of node select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    nodetype
        Label identifying type of nodes to consider when selecting:

        ALL - Select all nodes of the selected elements (default).

        ACTIVE - Select only the active nodes. An active node is a node that contributes DOFs to
                 the model.

        INACTIVE - Select only inactive  nodes (such as orientation or radiation).

        CORNER - Select only corner nodes.

        MID - Select only midside nodes.

        POS - Select nodes in position Num.

        FACE - Select nodes on face Num.

    num
        Position or face number for NodeType = POS or FACE.

    Notes
    -----
    NSLE selects NodeType nodes attached to the currently-selected set of
    elements. Only nodes on elements in the currently-selected element set
    can be selected.

    Note:: : When using degenerate hexahedral elements, NSLE, U,CORNER and
    NSLE,S,MID will not select the same set of nodes because some nodes
    appear as both corner and midside nodes.

    This command is valid in any processor.
    """
    command = "NSLE,%s,%s,%s" % (str(type_), str(nodetype), str(num))
    return self.run(command, **kwargs)


def nslk(self, type_="", **kwargs):
    """Selects those nodes associated with the selected keypoints.

    APDL Command: NSLK

    Parameters
    ----------
    type\_
        Label identifying the type of node select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    Notes
    -----
    Valid only if the nodes were generated by a keypoint meshing operation
    [KMESH, LMESH, AMESH, VMESH] on a solid model that contains the
    selected keypoints.

    This command is valid in any processor.
    """
    command = "NSLK,%s" % (str(type_))
    return self.run(command, **kwargs)


def nsll(self, type_="", nkey="", **kwargs):
    """Selects those nodes associated with the selected lines.

    APDL Command: NSLL

    Parameters
    ----------
    type\_
        Label identifying the type of node select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    nkey
        Specifies whether only interior line nodes are to be selected:

        0 - Select only nodes interior to selected lines.

        1 - Select all nodes (interior to line and at keypoints)
            associated with the selected lines.

    Notes
    -----
    Valid only if the nodes were generated by a line meshing
    operation [LMESH, AMESH, VMESH] on a solid model that contains
    the associated lines.

    This command is valid in any processor.
    """
    command = "NSLL,%s,%s" % (str(type_), str(nkey))
    return self.run(command, **kwargs)


def nslv(self, type_="", nkey="", **kwargs):
    """Selects those nodes associated with the selected volumes.

    APDL Command: NSLV

    Parameters
    ----------
    type\_
        Label identifying the type of node select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    nkey
        Specifies whether only interior volume nodes are to be selected:

        0 - Select only nodes interior to selected volumes.

        1 - Select all nodes (interior to volume, interior to areas, interior to lines, and
            at keypoints) associated with the selected volumes.

    Notes
    -----
    Valid only if the nodes were generated by a volume meshing operation
    [VMESH] on a solid model that contains the selected volumes.

    This command is valid in any processor.
    """
    command = "NSLV,%s,%s" % (str(type_), str(nkey))
    return self.run(command, **kwargs)


def partsel(self, type_="", pmin="", pmax="", pinc="", **kwargs):
    """Selects a subset of parts in an explicit dynamic analysis.

    APDL Command: PARTSEL

    Parameters
    ----------
    type\_
        Label identifying type of select. Because PARTSEL is a command
        macro, the label must be enclosed in single quotes.

        'S' - Select a new set (default).

        'R' - Reselect a set from the current set.

        'A' - Additionally select a set and extend the current set.

        'U' - Unselect a set from the current set.

        'ALL' - Select all parts.

        'NONE' - Unselect all parts.

        'INVE' - Invert the current selected set.

    Notes
    -----
    PARTSEL invokes an ANSYS macro that selects parts in an explicit
    dynamic analysis. When PARTSEL is executed, an element component is
    automatically created for each existing part. For example, the elements
    that make up PART 1 are grouped into the element component
    _PART1. Each time the PARTSEL command is executed, components
    for unselected parts will be unselected. To plot selected parts,
    choose Utility Menu> Plot> Parts in the GUI or issue the
    command PARTSEL,'PLOT'.

    After selecting parts, if you change the selected set of nodes or
    elements and then plot parts, the nodes and elements associated with
    the previously selected parts (from the last PARTSEL command) will
    become the currently selected set.

    Note:: : A more efficient way to select and plot parts is to use the
    ESEL (with ITEM = PART) and EPLOT commands. We recommend using ESEL
    instead of PARTSEL since PARTSEL will be phased out in a future
    release. Note that the menu path mentioned above for plotting parts
    does not work with the ESEL command; use Utility Menu> Plot> Elements
    instead.

    In an explicit dynamic small restart analysis (EDSTART,2), PARTSEL can
    be used to unselect a part during the solution even if it is referenced
    in some way (such as in a contact definition). (Note that ESEL cannot
    be used for this purpose.) However, in a new analysis or a full restart
    analysis (EDSTART,3), all parts that are used in some type of
    definition must be selected at the time of solution.

    This command is valid in any processor.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = "PARTSEL,%s,%s,%s,%s" % (
        str(type_), str(pmin), str(pmax), str(pinc))
    return self.run(command, **kwargs)


def vsel(self, type_="", item="", comp="", vmin="", vmax="", vinc="",
         kswp="", **kwargs):
    """Selects a subset of volumes.

    APDL Command: VSEL

    Parameters
    ----------
    type\_
        Label identifying the type of volume select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

        ALL - Restore the full set.

        NONE - Unselect the full set.

        INVE - Invert the current set (selected becomes unselected and vice versa).

        STAT - Display the current select status.

    Notes
    -----
    Selects volumes based on values of a labeled item and component.  For
    example, to select a new set of volumes based on volume numbers 1
    through 7, use VSEL,S,VOLU,,1,7.  The subset is used when the ALL label
    is entered (or implied) on other commands, such as VLIST,ALL.  Only
    data identified by volume number are selected.  Data are flagged as
    selected and unselected; no data are actually deleted from the
    database.

    This command is valid in any processor.

    For Selects based on non-integer numbers (coordinates, results, etc.),
    items that are within the range VMIN-Toler and VMAX+Toler are selected.
    The default tolerance Toler is based on the relative values of VMIN and
    VMAX as follows:

    If VMIN = VMAX, Toler = 0.005 x VMIN.

    If VMIN = VMAX = 0.0, Toler = 1.0E-6.

     If VMAX ≠ VMIN, Toler = 1.0E-8 x (VMAX-VMIN).

    Use the SELTOL command to override this default and specify Toler
    explicitly.

    Table: 251:: : VSEL - Valid Item and Component Labels
    """
    command = "VSEL,%s,%s,%s,%s,%s,%s,%s" % (str(type_), str(
        item), str(comp), str(vmin), str(vmax), str(vinc), str(kswp))
    return self.run(command, **kwargs)


def vsla(self, type_="", vlkey="", **kwargs):
    """Selects those volumes containing the selected areas.

    APDL Command: VSLA

    Parameters
    ----------
    type\_
        Label identifying the type of volume select:

        S - Select a new set (default).

        R - Reselect a set from the current set.

        A - Additionally select a set and extend the current set.

        U - Unselect a set from the current set.

    vlkey
        Specifies whether all contained volume areas must be selected
        [ASEL]:

        0 - Select volume if any of its areas are in the selected area set.

        1 - Select volume only if all of its areas are in the selected area set.

    Notes
    -----
    This command is valid in any processor.
    """
    command = "VSLA,%s,%s" % (str(type_), str(vlkey))
    return self.run(command, **kwargs)



from typing import Optional, Union

from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat


def areas(self, **kwargs):
    """Specifies "Areas" as the subsequent status topic.

    APDL Command: AREAS

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"AREAS,"
    return self.run(command, **kwargs)


def bool(self, **kwargs):
    """Specifies "Booleans" as the subsequent status topic.

    APDL Command: BOOL

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"BOOL,"
    return self.run(command, **kwargs)


def ceqn(self, **kwargs):
    """Specifies "Constraint equations" as the subsequent status topic.

    APDL Command: CEQN

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"CEQN,"
    return self.run(command, **kwargs)


def couple(self, **kwargs):
    """Specifies "Node coupling" as the subsequent status topic.

    APDL Command: COUPLE

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"COUPLE,"
    return self.run(command, **kwargs)


def digit(self, **kwargs):
    """Specifies "Node digitizing" as the subsequent status topic.

    APDL Command: DIGIT

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"DIGIT,"
    return self.run(command, **kwargs)


def elem(self, **kwargs) -> Optional[str]:
    """Specifies "Elements" as the subsequent status topic.

    APDL Command: ELEM

    Notes
    -----

    The STAT command should immediately follow this command,
    which should report the status for the specified topic.
    """
    command = f"ELEM,"
    return self.run(command, **kwargs)


def etype(self, **kwargs) -> Optional[str]:
    """Specify "Element types" as the subsequent status topic.

    APDL Command: ETYPE

    Examples
    --------
    >>> mapdl.et(1, 'SOLID186')
    >>> mapdl.etype()
    >>> mapdl.stat()
     ELEMENT TYPE        1 IS SOLID186     3-D 20-NODE STRUCTURAL SOLID
      KEYOPT( 1- 6)=        0      0      0        0      0      0
      KEYOPT( 7-12)=        0      0      0        0      0      0
      KEYOPT(13-18)=        0      0      0        0      0      0

    Notes
    -----
    This is a status [STAT] topic command.
    The STAT command should immediately follow this command,
    which should report the status for the specified topic.
    """
    return self.run("ETYPE", **kwargs)


def febody(self, **kwargs):
    """Specifies "Body loads on elements" as the subsequent status topic.

    APDL Command: FEBODY

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"FEBODY,"
    return self.run(command, **kwargs)


def fecons(self, **kwargs):
    """Specifies "Constraints on nodes" as the subsequent status topic.

    APDL Command: FECONS

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"FECONS,"
    return self.run(command, **kwargs)


def fefor(self, **kwargs):
    """Specifies "Forces on nodes" as the subsequent status topic.

    APDL Command: FEFOR

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"FEFOR,"
    return self.run(command, **kwargs)


def fesurf(self, **kwargs):
    """Specifies "Surface loads on elements" as the subsequent status topic.

    APDL Command: FESURF

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"FESURF,"
    return self.run(command, **kwargs)


def geometry(self, **kwargs):
    """Specifies "Geometry" as the subsequent status topic.

    APDL Command: GEOMETRY

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"GEOMETRY,"
    return self.run(command, **kwargs)


def keypts(self, **kwargs):
    """Specifies "Keypoints" as the subsequent status topic.

    APDL Command: KEYPTS

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"KEYPTS,"
    return self.run(command, **kwargs)


def line(self, **kwargs):
    """Specifies "Lines" as the subsequent status topic.

    APDL Command: LINE

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"LINE,"
    return self.run(command, **kwargs)


def mater(self, **kwargs):
    """Specifies "Material properties" as the subsequent status topic.

    APDL Command: MATER

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"MATER,"
    return self.run(command, **kwargs)


def meshing(self, **kwargs):
    """Specifies "Meshing" as the subsequent status topic.

    APDL Command: MESHING

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"MESHING,"
    return self.run(command, **kwargs)


def nodes(self, **kwargs):
    """Specifies "Nodes" as the subsequent status topic.

    APDL Command: NODES

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"NODES,"
    return self.run(command, **kwargs)


def prim(self, **kwargs):
    """Specifies "Solid model primitives" as the subsequent status topic.

    APDL Command: PRIM

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"PRIM,"
    return self.run(command, **kwargs)


def rcon(self, **kwargs):
    """Specifies "Real constants" as the subsequent status topic.

    APDL Command: RCON

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"RCON,"
    return self.run(command, **kwargs)


def selm(self, **kwargs):
    """Specifies "Superelements" as the subsequent status topic.

    APDL Command: SELM

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands
    are generated by the GUI and will appear in the log file
    (Jobname.LOG) if status is requested for some items under
    Utility Menu> List> Status.  This command will be immediately
    followed by a STAT command, which will report the status for
    the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    return self.run("SELM,", **kwargs)


def tble(self, **kwargs):
    """Specifies "Data table properties" as the subsequent status topic.

    APDL Command: TBLE

    Notes
    -----
    This is a status (STAT) topic command.  Status topic commands are
    generated by the GUI and will appear in the log file (Jobname.LOG) if
    status is requested for some items under Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"TBLE,"
    return self.run(command, **kwargs)


def volumes(self, **kwargs):
    """Specifies "Volumes" as the subsequent status topic.

    APDL Command: VOLUMES

    Notes
    -----
    This is a status [STAT] topic command.  Status topic commands are
    generated by the GUI and appear in the log file (Jobname.LOG) if status
    is requested for some items by choosing Utility Menu> List> Status.
    This command will be immediately followed by a STAT command, which will
    report the status for the specified topic.

    If entered directly into the program, the STAT command should
    immediately follow this command.
    """
    command = f"VOLUMES,"
    return self.run(command, **kwargs)



"""These SESSION commands are used to enter and exit the various
processors in the program.
"""


def aux2(self, **kwargs):
    """Enters the binary file dumping processor.

    APDL Command: /AUX2

    Notes
    -----
    Enters the binary file dumping processor (ANSYS auxiliary processor
    AUX2).  This processor is used to dump the contents of certain ANSYS
    binary files for visual examination.

    This command is valid only at the Begin Level.
    """
    command = "/AUX2,"
    return self.run(command, **kwargs)


def aux3(self, **kwargs):
    """Enters the results file editing processor.

    APDL Command: /AUX3

    Notes
    -----
    Enters the results file editing processor (ANSYS auxiliary processor
    AUX3).  This processor is used to edit ANSYS results files.

    This command is valid only at the Begin Level.
    """
    command = "/AUX3,"
    return self.run(command, **kwargs)


def aux12(self, **kwargs):
    """Enters the radiation processor.

    APDL Command: /AUX12

    Notes
    -----
    Enters the radiation processor (ANSYS auxiliary processor AUX12).  This
    processor supports the Radiation Matrix and the Radiosity Solver
    methods.

    This command is valid only at the Begin Level.
    """
    command = "/AUX12,"
    return self.run(command, **kwargs)


def aux15(self, **kwargs):
    """Enters the IGES file transfer processor.

    APDL Command: /AUX15

    Notes
    -----
    Enters the IGES file transfer processor (ANSYS auxiliary processor
    AUX15), used to read an IGES data file into the ANSYS program.

    This command is valid only at the Begin Level.
    """
    command = "/AUX15,"
    return self.run(command, **kwargs)


def finish(self, **kwargs):
    """Exits normally from a processor.

    APDL Command: FINISH

    Notes
    -----
    Exits any of the ANSYS processors or the DISPLAY program.  For the
    ANSYS processors, data will remain intact in the database but the
    database is not automatically written to a file (use the SAVE command
    to write the database to a file).  See also the /QUIT command for an
    alternate processor exit command.  If exiting POST1, POST26, or OPT,
    see additional notes below.

    POST1:  Data in the database will remain intact, including the POST1
    element table data, the path table data, the fatigue table data, and
    the load case pointers.

    POST26:  Data in the database will remain intact, except that POST26
    variables are erased and specification commands (such as FILE, PRTIME,
    NPRINT, etc.) are reset.  Use the /QUIT command to exit the processor
    and bypass these exceptions.

    This command is valid in any processor.  This command is not valid at
    the Begin level.
    """
    command = "FINISH,"
    return self.run(command, **kwargs)


def map(self, kdim="", kout="", limit="", **kwargs):
    """Maps pressures from source points to target surface elements.

    APDL Command: MAP

    Parameters
    ----------
    kdim
        Interpolation key:

        0 or 2 - Interpolation is done on a surface (default).

        3 - Interpolation is done within a volume. This option is
            useful if the supplied source data is volumetric field
            data rather than surface data.

    kout
        Key to control how pressure is applied when a target node is
        outside of the source region:

        0 - Use the pressure(s) of the nearest source point for
            target nodes outside of the region (default).

        1 - Set pressures outside of the region to zero.

    limit
        Number of nearby points considered for interpolation. The
        minimum is 5; the default is 20. Lower values reduce
        processing time.  However, some distorted or irregular
        meshes will require a higher LIMIT value to find the
        points encompassing the target node in order to define the
        region for interpolation.

    Notes
    -----
    Maps pressures from source points to target surface elements.
    """
    return self.run(f"MAP,,{kdim},,{kout},{limit}", **kwargs)


def post1(self, **kwargs):
    """Enters the database results postprocessor.

    APDL Command: /POST1

    Notes
    -----
    Enters the general database results postprocessor (POST1).  All load
    symbols (/PBC, /PSF, or /PBF) are automatically turned off with this
    command.

    This command is valid only at the Begin Level.
    """
    command = "/POST1,"
    return self.run(command, **kwargs)


def post26(self, **kwargs):
    """Enters the time-history results postprocessor.

    APDL Command: /POST26

    Notes
    -----
    Enters the time-history results postprocessor (POST26).

    This command is valid only at the Begin Level.
    """
    command = "/POST26,"
    return self.run(command, **kwargs)


def prep7(self, **kwargs):
    """Enters the model creation preprocessor.

    APDL Command: /PREP7

    Notes
    -----
    Enters the general input data preprocessor (PREP7).

    This command is valid only at the Begin Level.
    """
    command = "/PREP7,"
    return self.run(command, **kwargs)


def quit(self, **kwargs):
    """Exits a processor.

    APDL Command: /QUIT

    Notes
    -----
    This is an alternative to the FINISH command.  If any cleanup or file
    writing is normally done by the FINISH command, it is bypassed if the
    /QUIT command is used instead.  A new processor may be entered after
    this command.  See the /EXIT command to terminate the run.

    This command is valid in any processor.  This command is not valid at
    the Begin level.
    """
    command = "/QUIT,"
    return self.run(command, **kwargs)


def slashsolu(self, **kwargs):
    """Enters the solution processor.

    APDL Command: /SOLU

    Notes
    -----
    This command is valid only at the Begin Level.
    """
    command = "/SOLU,"
    return self.run(command, **kwargs)



def format(self, ndigit="", ftype="", nwidth="", dsignf="", line="",
           char="", **kwargs):
    """Specifies format controls for tables.

    APDL Command: /FORMAT

    Parameters
    ----------
    ndigit
        Number of digits (3 to 32) in first table column (usually the node
        or element number).  Initially defaults to 7.

    ftype
        FORTRAN format types (initially defaults to G):

        G - Gxx.yy.  xx and yy are described below.

        F - Fxx.yy

        E - Exx.yy

    nwidth
        Total width (9 to 32) of the field (the xx in Ftype).  Initially
        defaults to 12.

    dsignf
        Number of digits after the decimal point (yy in F or E format) or
        number of significant digits in G format.  Range is 2 to xx-7 for
        Ftype = G or E; and 0 to xx-4 for Ftype = F.  Initially defaults to
        5.

    line
        Number of lines (11 minimum) per page.  Defaults to ILINE or BLINE
        from the /PAGE command.

    char
        Number of characters (41 to 240, system-dependent) per line before
        wraparound.  Defaults to ICHAR or BCHAR from the /PAGE command.

    Notes
    -----
    Specifies various format controls for tables printed with the POST1
    PRNSOL, PRESOL, PRETAB, PRRSOL, PRPATH, and CYCCALC commands.  A blank
    (or out-of-range) field on the command retains the current setting.
    Issue /FORMAT,STAT to display the current settings.  Issue /FORMAT,DEFA
    to reestablish the initial default specifications.

    For the POST26 PRVAR command, the Ftype, NWIDTH, and DSIGNF fields
    control the time output format.

    This command is valid in any processor.
    """
    command = f"/FORMAT,{ndigit},{ftype},{nwidth},{dsignf},{line},{char}"
    return self.run(command, **kwargs)


def header(self, header="", stitle="", idstmp="", notes="", colhed="",
           minmax="", **kwargs):
    """Sets page and table heading print controls.

    APDL Command: /HEADER

    Parameters
    ----------
    header
        ANSYS page header (system, date, time, version, copyright, title,
        etc.):

        ON - Turns this item on (default for batch mode; not available for interactive
             mode).

        OFF - Turns this item off.

        (blank) - Retains the previous setting.

    stitle
        Subtitles (see /STITLE command):   ON, OFF, or (blank) (see above).

    idstmp
        Load step information (step number, substep number, time value):
        ON, OFF, or (blank) (see above).

    notes
        Information relative to particular table listings:   ON, OFF, or
        (blank) (see above).

    colhed
        Column header labels of table listings (currently only for single
        column tables):   ON, OFF, or (blank) (see above).

    minmax
        Minimum/maximum information or totals after table listings:   ON,
        OFF, or (blank) (see above).

    Notes
    -----
    Sets specifications on or off for page and table heading print controls
    associated with the POST1 PRNSOL, PRESOL, PRETAB, PRRSOL, and PRPATH
    commands.

    Note:: : If the printout caused a  top-of-form (page eject to top of
    next page), the top-of-form is also suppressed with the printout.
    Issue /HEADER,STAT to display the current settings.  Issue /HEADER,DEFA
    to reset the default specifications.

    This command is valid in any processor.
    """
    command = f"/HEADER,{header},{stitle},{idstmp},{notes},{colhed},{minmax}"
    return self.run(command, **kwargs)


def irlist(self, **kwargs):
    """Prints inertia relief summary table.

    APDL Command: IRLIST

    Notes
    -----
    Prints the inertia relief summary data, including the mass summary
    table, the total load summary table, and the inertia relief summary
    table resulting from the inertia relief calculations. These
    calculations are performed in the solution phase [SOLVE] as specified
    by the IRLF command.

    Inertia relief output is stored in the database rather than in the
    results file (Jobname.RST). When you issue IRLIST, ANSYS pulls the
    information from the database, which contains the inertia relief output
    from the most recent solution [SOLVE].

    This command is valid in any processor.
    """
    command = f"IRLIST,"
    return self.run(command, **kwargs)


def page(self, iline="", ichar="", bline="", bchar="", comma="", **kwargs):
    """Defines the printout and screen page size.

    APDL Command: /PAGE

    Parameters
    ----------
    iline
        Number of lines (11 minimum) per "page" or screen.  Defaults to 24.
        Applies to interactive non-GUI to the screen output only.

    ichar
        Number of characters (41 to 132) per line before wraparound.
        Defaults to 80.  Applies to interactive non-GUI to the screen
        output only.

    bline
        Number of lines (11 minimum) per page.  Defaults to 56.  Applies to
        batch mode [/BATCH], diverted [/OUTPUT], or interactive GUI [/MENU]
        output. If negative, no page headers are output.

    bchar
        Number of characters (41 to 240 (system dependent)) per line before
        wraparound.  Defaults to 132.  Applies to batch mode [/BATCH],
        diverted [/OUTPUT], or interactive GUI [/MENU] output.

    comma
        Input 1 to specify comma-separated output for node [NLIST] and
        element [ELIST] output.

    Notes
    -----
    Defines the printout page size for batch runs and the screen page size
    for interactive runs.  Applies to the POST1 PRNSOL, PRESOL, PRETAB,
    PRRSOL, and PRPATH commands.  See the /HEADER command for additional
    controls (page ejects, headers, etc.) that affect the amount of
    printout.  A blank (or out-of-range) value retains the previous
    setting.  Issue /PAGE,STAT to display the current settings.  Issue
    /PAGE,DEFA to reset the default specifications.

    This command is valid in any processor.
    """
    command = f"/PAGE,{iline},{ichar},{bline},{bchar},{comma}"
    return self.run(command, **kwargs)


def prerr(self, **kwargs):
    """Prints SEPC and TEPC.

    APDL Command: PRERR

    Notes
    -----
    Prints the percent error in structural energy norm (SEPC) and the
    thermal energy norm percent error (TEPC).  Approximations of mesh
    discretization error associated with a solution are calculated for
    analyses having structural or thermal degrees of freedom.

    The structural approximation is based on the energy error (which is
    similar in concept to the strain energy) and represents the error
    associated with the discrepancy between the calculated stress field and
    the globally continuous stress field (see POST1 - Error Approximation
    Technique in the Mechanical APDL Theory Reference).  This discrepancy
    is due to the assumption in the elements that only the displacements
    are continuous at the nodes.  The stress field is calculated from the
    displacements and should also be continuous, but generally is not.

    Thermal analyses may use any solid and shell thermal element having
    only temperature degrees of freedom.  The thermal approximation is
    based on the total heat flow dissipation and represents the error
    associated with the discrepancy between the calculated nodal thermal
    flux within an element and a continuous global thermal flux.  This
    continuous thermal flux is calculated with the normal nodal averaging
    procedure.

    The volume (result label VOLU) is used to calculate the energy error
    per element (result label SERR for the structural energy error and TERR
    for the thermal energy error).  These energy errors, along with the
    appropriate energy, are then used to calculate the percent error in
    energy norm (SEPC for structural and TEPC for thermal). These
    percentages can be listed by the PRERR command, retrieved by the *GET
    command (with labels SEPC and TEPC) for further calculations, and shown
    on the displacement display (PLDISP), as applicable.

    For structural analyses, the maximum absolute value of nodal stress
    variation of any stress component for any node of an element (result
    item SDSG) is also calculated.  Similarly, for thermal gradient
    components, TDSG is calculated.  Minimum and maximum result bounds
    considering the possible effect of discretization error will be shown
    on contour displays (PLNSOL).  For shell elements, the top surface
    location is used to produce a meaningful percentage value.  SERR, TERR,
    SEPC, TEPC, SDSG, and TDSG will be updated whenever the nodal stresses
    or fluxes are recalculated.

    If the energy error is a significant portion of the total energy, then
    the analysis should be repeated using a finer mesh to obtain a more
    accurate solution.  The energy error is relative from problem to
    problem but will converge to a zero energy error as the mesh is
    refined.  An automated adaptive meshing procedure using this energy
    error is described with the ADAPT macro.

    The following element- and material-type limitations apply:

    Valid with most 2-D solid, 3-D solid, axisymmetric solid, or 3-D shell
    elements.

    The following element types are not valid: SHELL28, SHELL41, and
    SOLID65.

    The model should have only structural or thermal degrees of freedom.

    The analysis must be linear (for both material and geometry).

    Multi-material (for example, composite) elements are not valid.

    Transition regions from one material to another are not valid (that is,
    the entire model should consist of one material).

    Anisotropic materials (TB,ANEL) are not considered.
    """
    command = f"PRERR,"
    return self.run(command, **kwargs)


def priter(self, **kwargs):
    """Prints solution summary data.

    APDL Command: PRITER

    Notes
    -----
    Prints solution summary data (such as time step size, number of
    equilibrium iterations, convergence values, etc.) from a static or full
    transient analysis. All other analyses print zeros for the data.
    """
    command = f"PRITER,"
    return self.run(command, **kwargs)



def compress(self, **kwargs):
    """Deletes all specified sets.

    APDL Command: COMPRESS

    Notes
    -----
    Issue this command to delete all sets specified with the DELETE
    command.
    """
    command = f"COMPRESS,"
    return self.run(command, **kwargs)


def list(self, level="", **kwargs):
    """Lists out the sets in the results file.

    APDL Command: LIST

    Notes
    -----
    This command lists the results set number, the load step, substep, and
    time step for each set. It also shows all sets marked for deletion.
    """
    command = "LIST,%s" % (str(level))
    return self.run(command, **kwargs)


def fileaux3(self, fname="", ext="", **kwargs):
    """Specifies the results file to be edited.

    APDL Command: FILEAUX3

    Parameters
    ----------
    fname
        File name and directory path (248 characters maximum,
        including the characters needed for the directory path).
        An unspecified directory path defaults to the working
        directory; in this case, you can use all 248 characters
        for the file name.

    ext
        Filename extension (eight-character maximum).

    Notes
    -----
    Specifies the results file to be edited.
    """
    command = f"FILEAUX3,{fname},{ext}"
    return self.run(command, **kwargs)


def modify(self, set_="", lstep="", iter_="", cumit="", time="", ktitle="",
           **kwargs):
    """Changes the listed values of the data in a set.

    APDL Command: MODIFY

    Parameters
    ----------
    set\_
        Set of data in results file to be modified.

    lstep
        The new load step number.

    iter\_
        The new load substep number.

    cumit
        The new cumulative iteration.

    time
        The new time/frequency value.

    ktitle
        Indicates if the set title should be modified.

        0 - Keep the original title.

        1 - Change the title to the title specified with the most current /TITLE command.

    Notes
    -----
    Use this command to change the listed values in a data set in a results
    file. Using this command does not change any actual model data; it
    affects only the values listed in the results file.

    For example, if you start with the following results file:

    and you then issue the following commands:

    The modified results file would look like this:
    """
    command = f"MODIFY,{set_},{lstep},{iter_},{cumit},{time},{ktitle}"
    return self.run(command, **kwargs)


def undelete(self, option="", nstart="", nend="", **kwargs):
    """Removes results sets from the group of sets selected for editing.

    APDL Command: UNDELETE

    Parameters
    ----------
    option
        Specifies which sets are to be removed from the selected sets.

        SET - Specifies one or more particular sets in the results file that are to be
              removed from the group of sets selected for deletion.

        ALL - Removes all selected sets that are currently selected for deletion.

    nstart
        The first set to be removed from the set selected for deletion.

    nend
        The final set to be removed from the set selected for deletion.
        This field is used only if operating on more than one sequential
        set.

    Notes
    -----
    Use this command if you have previously marked a set for deletion (with
    the DELETE command) and now wish to keep that set instead of deleting
    it.
    """
    command = f"UNDELETE,{option},{nstart},{nend}"
    return self.run(command, **kwargs)


def delete(self, set_="", nstart="", nend="", **kwargs):
    """Specifies sets in the results file to be deleted before postprocessing.

    APDL Command: DELETE

    Parameters
    ----------
    set_
        Specifies that sets in the results file are to be deleted.

    nstart
        The first set in a results file to be deleted.

    nend
        The final set in a results file to be deleted. This field is used
        only if deleting more than one sequential sets.

    Notes
    -----
    DELETE is a specification command that flags sets in the results
    file for deletion. It should be followed by a COMPRESS command,
    the corresponding action command that deletes the specified sets.

    The DELETE command is valid only in the results file editing processor
    (ANSYS auxiliary processor AUX3).
    """
    command = f"DELETE,{set_},{nstart},{nend}"
    return self.run(command, **kwargs)



def mflcomm(self, type_="", fname1="", intname1="", label1="", fname2="",
            intname2="", label2="", option="", **kwargs):
    """Defines a load transfer for code coupling analyses.

    APDL Command: MFLCOMM

    Parameters
    ----------
    type\_
         Set to SURF for a surface load transfer. Only surface load
        transfers are available for MFX.

    fname1
        Sets the field solver name for the server (sending) code with a
        case-sensitive character string of up to 80 characters.

    intname1
        Sets the interface name or number for the field solver of the
        server code. ANSYS interfaces are numbered and are defined by the
        SF family of commands (SF, SFA, or SFE) with the FSIN surface load
        label. CFX interfaces use names, which are set in CFX-Pre.

    label1
        Sets the surface load label for the field solver of the server code
        with a character string of up to 80 characters. ANSYS uses a
        combination of the label and option to determine what data is
        transferred (e.g., heat flows and not fluxes are sent with the
        label/option pair HFLU/CPP). ANSYS cannot serve total force or
        total force density to CFX for either formulation. CFX will send
        the data requested by the label regardless of the option. CFX
        labels that have more than one word must be enclosed in single
        quotes. Note that this field is case-sensitive; i.e., FORC will
        work, but forc will not.

    fname2
        Sets the field solver name for the client (receiving) code with a
        character string of up to 80 characters.

    intname2
        Sets the interface name or number for the field solver of the
        client code with a character string of up to 80 characters. ANSYS
        interfaces are numbered and are defined by the SF family of
        commands (SF, SFA, or SFE) with the FSIN surface load label. CFX
        interfaces use names, which are set in CFX-Pre.

    label2
        Sets the surface load label for the field solver of the client code
        with a character string of up to 80 characters. ANSYS uses a
        combination of the label and option to determine what data is
        transferred (e.g., heat flows and not fluxes are sent with the
        label-option pair HFLU/CPP). CFX will send the data requested by
        the label regardless of the option. CFX labels that have more than
        one word must be enclosed in single quotes. Note that this field is
        case-sensitive; i.e., FORC will work, but forc will not.

    option
        NONC

        NONC - Profile preserving: Sets the interface load transfer to the nonconservative
               formulation (default for displacement and temperature). In
               the nonconservative formulation, the force density (or heat
               flux) is transferred across the interface, preserving the
               density profile between the two fields.

        CPP - Conservative: Uses a local conservative formulation while preserving the
              density profile (default for total force and wall heat flow).
              In the conservative formulation, total force (or heat flow)
              must be transferred across the interface from the CFX field
              solver to the ANSYS field solver.

    Notes
    -----
    ANSYS input should always be in consistent units for its model.

    ANSYS uses a combination of the label and option to determine what data
    to transfer. CFX will send exactly the data requested by the label,
    regardless of the option. However, for the NONC option, the CFX label
    must be Total Force Density or Wall Heat Flux and for the CPP option,
    the CFX label must be Total Force or Wall Heat Flow.

    For more information on profile preserving and conservative load
    transfer, see Load Interpolation in the Coupled-Field Analysis Guide.
    Mapping Diagnostics are also available; however, if the improperly-
    mapped nodes are based on the CFX mesh, you should ignore the ANSYS-
    generated components because the CFX nodes are not present in the ANSYS
    database.

    If you are working interactively, you can choose two pre-defined
    combinations, Mechanical or Thermal, or you can choose a Custom option.
    If you choose the Mechanical load type, then the Total Force Density
    and Total Mesh Displacement data (corresponding to the ANSYS FORC and
    DISP labels, respectively) is transferred. If you choose the Thermal
    load type, then the Temperature and Wall Heat Flux data (corresponding
    to the ANSYS TEMP and HFLU labels, respectively) is transferred. If you
    choose Custom, you can select any valid combination of label and option
    as described above.

    The ANSYS Multi-field solver solver does not allow you to switch the
    load transfer direction for the same load quantity across the same
    interfaces for a restart run. For example, if Field1 sends temperature
    to and receives heat flow from Field2 across Interface 1 in a previous
    solution, then you cannot make Field1 send heat flow to and receive
    temperatures from Field2 across the same interface in a restart run,
    even if you cleared the corresponding load transfer command.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFLCOMM,{type_},{fname1},{intname1},{label1},{fname2},{intname2},{label2},{option}"
    return self.run(command, **kwargs)


def mfsurface(self, inumb="", fnumb1="", label="", fnumb2="", **kwargs):
    """Defines a surface load transfer for an ANSYS Multi-field solver

    APDL Command: MFSURFACE
    analysis.

    Parameters
    ----------
    inumb
        Interface number for load transfer. The interface number
        corresponds to the interface number specified by the surface flag
        FSIN (SFxxcommands).

    fnumb1
        Field number of sending field.

    label
        Valid surface load labels:

    fnumb2
        Field number for receiving field.

    Notes
    -----
    This command is also valid in PREP7.

    The ANSYS Multi-field solver solver does not allow you to switch the
    load transfer direction for the same load quantity across the same
    interfaces for a restart run. For example, if Field1 sends temperature
    to and receives heat flow from Field2 across Interface 1 in a previous
    solution, then you cannot make Field1 send heat flow to and receive
    temperatures from Field2 across the same interface in a restart run,
    even if you cleared the corresponding load transfer command.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFSURFACE,{inumb},{fnumb1},{label},{fnumb2}"
    return self.run(command, **kwargs)


def mfvolume(self, inumb="", fnumb1="", label="", fnumb2="", **kwargs):
    """Defines a volume load transfer for an ANSYS Multi-field solver

    APDL Command: MFVOLUME
    analysis.

    Parameters
    ----------
    inumb
        Interface number for load transfer. The interface number
        corresponds to the interface number specified by the volume flag
        FVIN (BFE command).

    fnumb1
        Field number of sending field.

    label
        Valid volume load labels:

    fnumb2
        Field number for receiving field.

    Notes
    -----
    This command is also valid in PREP7.

    The ANSYS Multi-field solver solver does not allow you to switch the
    load transfer direction for the same load quantity across the same
    interfaces for a restart run. For example, if Field1 sends temperature
    to and receives heat flow from Field2 across Interface 1 in a previous
    solution, then you cannot make Field1 send heat flow to and receive
    temperatures from Field2 across the same interface in a restart run,
    even if you cleared the corresponding load transfer command.

    See Multi-field Commands in the Coupled-Field Analysis Guide for a list
    of all ANSYS Multi-field solver commands and their availability for MFS
    and MFX analyses.

    Distributed ANSYS Restriction: This command is not supported in
    Distributed ANSYS.
    """
    command = f"MFVOLUME,{inumb},{fnumb1},{label},{fnumb2}"
    return self.run(command, **kwargs)



from typing import Optional, Union

from ansys.mapdl.core.mapdl_types import MapdlInt, MapdlFloat
from ansys.mapdl.core._commands import parse


def extopt(self, lab: str = "", val1: Union[str, int] = "",
           val2: Union[str, int] = "", val3: MapdlInt = "",
           val4: MapdlInt = "", **kwargs) -> Optional[str]:
    """Controls options relating to the generation of volume elements from area elements.

    APDL Command: EXTOPT

    Parameters
    ----------
    lab
        Label identifying the control option. The meanings of
        Val1, Val2, and Val3 will vary depending on Lab.

        ON - Sets carryover of the material attributes, real
             constant attributes, and element coordinate system
             attributes of the pattern area elements to the
             generated volume elements.  Sets the pattern
             area mesh to clear when volume generations are done.
             Val1, Val2, and Val3 are ignored.

        OFF - Removes all settings associated with this command.
              Val1, Val2, and Val3 are ignored.

        STAT - Shows all settings associated with this command.
               Val1, Val2, Val3, and Val4 are ignored.

        ATTR - Sets carryover of particular pattern area attributes
               (materials, real constants, and element coordinate
               systems) of the pattern area elements to the
               generated volume elements. (See 2.) Val1 can be:

            0 - Sets volume elements to use
                current MAT command settings.

            1 - Sets volume elements to use material
                attributes of the pattern area elements.

        Val2 can be:

            0 - Sets volume elements to use current REAL
                command settings.

            1 - Sets volume elements to use real constant attributes
                of the pattern area elements.

        Val3 can be:

            0 - Sets volume elements to use current ESYS command
                settings.

            1 - Sets volume elements to use element coordinate
                system attributes of the pattern area elements.

        Val4 can be:

            0 - Sets volume elements to use current SECNUM command
                settings.

            1 - Sets volume elements to use section attributes of
                the pattern area elements.

        ESIZE - Val1 sets the number of element divisions in the
                direction of volume generation or volume sweep.
                For VDRAG and VSWEEP, Val1 is overridden by the
                LESIZE command NDIV setting. Val2 sets the spacing
                ratio (bias) in the direction of volume generation
                or volume sweep. If positive, Val2 is the nominal
                ratio of last division size to first division size
                (if > 1.0, sizes increase, if < 1.0, sizes
                decrease). If negative, Val2 is the nominal ratio of
                center division(s) size to end divisions size. Ratio
                defaults to 1.0 (uniform spacing).
                Val3 and Val4 are ignored.

        ACLEAR - Sets clearing of pattern area mesh.
                 (See 3.) Val1 can be:

            0 - Sets pattern area to remain meshed when volume
                generation is done.

            1 - Sets pattern area mesh to clear when volume
                generation is done. Val2, Val3, and Val4 are
                ignored.

        VSWE - Indicates that volume sweeping options will be set
               using Val1 and Val2. Settings specified with EXTOPT,
               VSWE will be used the next time the VSWEEP command
               is invoked. If Lab = VSWE, Val1 becomes a label.
               Val1 can be:

        AUTO - Indicates whether you will be prompted for the source
               and target used by VSWEEP or if VSWE should
               automatically determine the source and target.
               If Val1 = AUTO, Val2 is ON by default. VSWE will
               automatically determine the source and target for
               VSWEEP. You will be allowed to pick more than one
               volume for sweeping. When Val2 = OFF, the user will
               be prompted for the source and target for VSWEEP.
               You will only be allowed to pick one volume for
               sweeping.

        TETS - Indicates whether VSWEEP will tet mesh non-sweepable
               volumes or leave them unmeshed. If Val1 = TETS,
               Val2 is OFF by default. Non-sweepable volumes will be
               left unmeshed. When Val2 = ON, the non-sweepable
               volumes will be tet meshed if the assigned element
               type supports tet shaped elements.

    val1, val2, val3, val4
        Additional input values as described under each option for
        Lab.

    Notes
    -----
    EXTOPT controls options relating to the generation of volume
    elements from pattern area elements using the VEXT, VROTAT,
    VOFFST, VDRAG, and VSWEEP commands.  (When using VSWEEP,
    the pattern area is referred to as the source area.)

    Enables carryover of the attributes  of the pattern area
    elements to the generated volume elements when you are using
    VEXT, VROTAT, VOFFST, or VDRAG. (When using VSWEEP, since the
    volume already exists, use the VATT command to assign attributes
    before sweeping.)

    When you are using VEXT, VROTAT, VOFFST, or VDRAG, enables
    clearing of the pattern area mesh when volume generations are
    done. (When you are using VSWEEP, if selected, the area meshes
    on the pattern (source), target, and/or side areas clear when
    volume sweeping is done.)

    Neither EXTOPT,VSWE,AUTO nor EXTOPT,VSWE,TETS will be affected
    by EXTOPT,ON or EXTOPT, OFF.
    """
    command = f"EXTOPT,{lab},{val1},{val2},{val3},{val4}"
    return self.run(command, **kwargs)


def v(self, p1="", p2="", p3="", p4="", p5="", p6="", p7="", p8="",
      **kwargs) -> int:
    """Define a volume through keypoints.

    APDL Command: V

    Parameters
    ----------
    p1 : int, optional
        Keypoint defining starting corner of volume.

    p2 : int, optional
        Keypoint defining second corner of volume.

    p3 : int, optional
        Keypoint defining third corner of volume.

    p4 : int, optional
        Keypoint defining fourth corner of volume.

    p5 : int, optional
        Keypoint defining fifth corner of volume.

    p6 : int, optional
        Keypoint defining sixth corner of volume.

    p7 : int, optional
        Keypoint defining seventh corner of volume.

    p8 : int, optional
        Keypoint defining eighth corner of volume.

    Returns
    -------
    int
        Volume number of the generated volume.

    Examples
    --------
    Create a simple cube volume.

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 1, 0, 0)
    >>> k2 = mapdl.k("", 1, 1, 0)
    >>> k3 = mapdl.k("", 0, 1, 0)
    >>> k4 = mapdl.k("", 0, 0, 1)
    >>> k5 = mapdl.k("", 1, 0, 1)
    >>> k6 = mapdl.k("", 1, 1, 1)
    >>> k7 = mapdl.k("", 0, 1, 1)
    >>> v0 = mapdl.v(k0, k1, k2, k3, k4, k5, k6, k7)
    >>> v0
    1

    Create a triangular prism

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 1, 0, 0)
    >>> k2 = mapdl.k("", 1, 1, 0)
    >>> k3 = mapdl.k("", 0, 1, 0)
    >>> k4 = mapdl.k("", 0, 0, 1)
    >>> k5 = mapdl.k("", 1, 0, 1)
    >>> k6 = mapdl.k("", 1, 1, 1)
    >>> k7 = mapdl.k("", 0, 1, 1)
    >>> v1 = mapdl.v(k0, k1, k2, k2, k4, k5, k6, k6)
    >>> v1
    2

    Create a tetrahedron

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 1, 0, 0)
    >>> k2 = mapdl.k("", 1, 1, 0)
    >>> k3 = mapdl.k("", 0, 0, 1)
    >>> v2 = mapdl.v(k0, k1, k2, k2, k3, k3, k3, k3)
    >>> v2
    3

    Notes
    -----
    Defines a volume (and its corresponding lines and areas)
    through eight (or fewer) existing keypoints.  Keypoints must
    be input in a continuous order.  The order of the keypoints
    should be around the bottom and then the top.  Missing lines
    are generated "straight" in the active coordinate system and
    assigned the lowest available numbers [NUMSTR].  Missing areas
    are generated and assigned the lowest available numbers.

    Solid modeling in a toroidal coordinate system is not recommended.

    Certain faces may be condensed to a line or point by repeating
    keypoints.   For example, use V,P1,P2,P3,P3,P5,P6,P7,P7   for a
    triangular prism or V,P1,P2,P3,P3,P5,P5,P5,P5  for a tetrahedron.

    Using keypoints to produce partial sections in CSYS = 2 can generate
    anomalies; check the resulting volumes carefully.
    """
    command = f"V,{p1},{p2},{p3},{p4},{p5},{p6},{p7},{p8}"
    return parse.parse_v(self.run(command, **kwargs))


def va(self, a1="", a2="", a3="", a4="", a5="", a6="", a7="", a8="", a9="",
       a10="", **kwargs) -> int:
    """Generate a volume bounded by existing areas.

    APDL Command: VA

    Parameters
    ----------
    a1, a2, a3, . . . , a10
        List of areas defining volume.  The minimum number of
        areas is 4.  If A1 = ALL, use all selected [ASEL] areas
        and ignore A2 to A10.  A component name may also be
        substituted for A1.

    Returns
    -------
    int
        Volume number of the volume.


    Examples
    --------
    Create a simple tetrahedral bounded by 4 areas.

    >>> k0 = mapdl.k('', -1, 0, 0)
    >>> k1 = mapdl.k('', 1, 0,  0)
    >>> k2 = mapdl.k('', 1, 1, 0)
    >>> k3 = mapdl.k('', 1, 0.5, 1)
    >>> a0 = mapdl.a(k0, k1, k2)
    >>> a1 = mapdl.a(k0, k1, k3)
    >>> a2 = mapdl.a(k1, k2, k3)
    >>> a3 = mapdl.a(k0, k2, k3)
    >>> vnum = mapdl.va(a0, a1, a2, a3)
    >>> vnum
    1

    Notes
    -----
    This command conveniently allows generating volumes from
    regions having more than eight keypoints (which is not allowed
    with the V command).  Areas may be input in any order.  The
    exterior surface of a VA volume must be continuous, but holes
    may pass completely through it.
    """
    command = f"VA,{a1},{a2},{a3},{a4},{a5},{a6},{a7},{a8},{a9},{a10}"
    return parse.parse_v(self.run(command, **kwargs))


def vdele(self, nv1="", nv2="", ninc="", kswp="", **kwargs):
    """Deletes unmeshed volumes.

    APDL Command: VDELE

    Parameters
    ----------
    nv1, nv2, ninc
        Delete volumes from NV1 to NV2 (defaults to NV1) in steps of NINC
        (defaults to 1).  If NV1 = ALL, NV2 and NINC are ignored and all
        selected volumes [VSEL] are deleted.  If NV1 = P, graphical picking
        is enabled and all remaining command fields are ignored (valid only
        in the GUI).  A component name may also be substituted for NV1 (NV2
        and NINC are ignored).

    kswp
        Specifies whether keypoints, lines, and areas are also deleted:

        0 - Delete volumes only (default).

        1 - Delete volumes, as well as keypoints, lines, and areas attached to the
            specified volumes but not shared by other volumes.
    """
    command = f"VDELE,{nv1},{nv2},{ninc},{kswp}"
    return self.run(command, **kwargs)


def vdgl(self, nv1="", nv2="", ninc="", **kwargs):
    """Lists keypoints of a volume that lie on a parametric degeneracy.

    APDL Command: VDGL

    Parameters
    ----------
    nv1, nv2, ninc
        List keypoints that lie on a parametric degeneracy on volumes from
        NV1 to NV2 (defaults to NV1) in steps of NINC (defaults to 1).  If
        NV1 = ALL (default), NV2 and NINC will be ignored and keypoints on
        all selected volumes [VSEL] will be listed.  If NV1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  You may also substitute a component name
        for NV1 (ignore NV2 and NINC).

    Notes
    -----
    See the Modeling and Meshing Guide for details about parametric
    degeneracies.

    This command is valid in any processor.
    """
    command = f"VDGL,{nv1},{nv2},{ninc}"
    return self.run(command, **kwargs)


def vdrag(self, na1="", na2="", na3="", na4="", na5="", na6="", nlp1="",
          nlp2="", nlp3="", nlp4="", nlp5="", nlp6="", **kwargs) -> str:
    """Generate volumes by dragging an area pattern along a path.

    APDL Command: VDRAG

    Parameters
    ----------
    na1, na2, na3, . . . , na6
        List of areas in the pattern to be dragged (6 maximum if
        using keyboard entry).  If NA1 = ALL, all selected areas
        will be swept along the path.  A component name may also
        be substituted for NA1.

    nlp1, nlp2, nlp3, . . . , nlp6
        List of lines defining the path along which the pattern is
        to be dragged (6 maximum if using keyboard entry).  Must
        be a continuous set of lines.  To be continuous, adjacent
        lines must share the connecting keypoint (the end keypoint
        of one line must also be first keypoint of the next line).

    Returns
    -------
    str
        MAPDL command output.

    Examples
    --------
    Create a square with a hole in it and drag it along an arc.

    >>> anum0 = mapdl.blc4(0, 0, 1, 1)
    >>> anum1 = mapdl.blc4(0.25, 0.25, 0.5, 0.5)
    >>> aout = mapdl.asba(anum0, anum1)
    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 1, 0, 1)
    >>> k2 = mapdl.k("", 1, 0, 0)
    >>> l0 = mapdl.larc(k0, k1, k2, 2)
    >>> output = mapdl.vdrag(aout, nlp1=l0)
    >>> print(output)
    DRAG AREAS
      3,
    ALONG LINES
      9


    Notes
    -----
    Generates volumes (and their corresponding keypoints, lines,
    and areas) by sweeping a given area pattern along a
    characteristic drag path.  If the drag path consists of
    multiple lines, the drag direction is determined by the
    sequence in which the path lines are input (NLP1, NLP2, etc.).
    If the drag path is a single line (NLP1), the drag direction
    is from the keypoint on the drag line that is closest to the
    first keypoint of the given area pattern to the other end of
    the drag line.

    The magnitude of the vector between the keypoints of the given
    pattern and the first path keypoint remains constant for all
    generated keypoint patterns and the path keypoints.  The
    direction of the vector relative to the path slope also
    remains constant so that patterns may be swept around curves.
    Lines are generated with the same shapes as the given pattern
    and the path lines.

    Keypoint, line, area, and volume numbers are automatically
    assigned (beginning with the lowest available values
    [NUMSTR]).  Adjacent lines use a common keypoint, adjacent
    areas use a common line, and adjacent volumes use a common
    area.  For best results, the entities to be dragged should be
    orthogonal to the start of the drag path.  Drag operations
    that produce an error message may create some of the desired
    entities prior to terminating.

    If element attributes have been associated with the input area
    via the AATT command, the opposite area generated by the VDRAG
    operation will also have those attributes (i.e., the element
    attributes from the input area are copied to the opposite
    area).  Note that only the area opposite the input area will
    have the same attributes as the input area; the areas adjacent
    to the input area will not.

    If the input areas are meshed or belong to a meshed volume,
    the area(s) can be extruded to a 3-D mesh.  Note that the NDIV
    argument of the ESIZE command should be set before extruding
    the meshed areas.  Alternatively, mesh divisions can be
    specified directly on the drag line(s) (LESIZE).  See the
    Modeling and Meshing Guide for more information.

    You can use the VDRAG command to generate 3-D interface
    element meshes for elements INTER194 and INTER195. When
    generating interface element meshes using VDRAG, you must
    specify the line divisions to generate one interface element
    directly on the drag line using the LESIZE command.  The
    source area to be extruded becomes the bottom surface of the
    interface element. Interface elements must be extruded in what
    will become the element's local x direction, that is, bottom
    to top.
    """
    command = f"VDRAG,{na1},{na2},{na3},{na4},{na5},{na6},{nlp1},{nlp2},{nlp3},{nlp4},{nlp5},{nlp6}"
    return self.run(command, **kwargs)


def vext(self, na1="", na2="", ninc="", dx="", dy="", dz="", rx="", ry="",
         rz="", **kwargs) -> str:
    """Generate additional volumes by extruding areas.

    APDL Command: VEXT

    Parameters
    ----------
    na1, na2, ninc
        Set of areas (NA1 to NA2 in steps of NINC) that defines
        the pattern to be extruded.  NA2 defaults to NA1, NINC
        defaults to 1.  If NA1 = ALL, NA2 and NINC are ignored and
        the pattern is defined by all selected areas.  A component
        name may also be substituted for NA1 (NA2 and NINC are
        ignored).

    dx, dy, dz
        Increments to be applied to the X, Y, and Z keypoint
        coordinates in the active coordinate system (DR, Dθ, DZ
        for cylindrical; DR, Dθ, DΦ for spherical).

    rx, ry, rz
        Scale factors to be applied to the X, Y, and Z keypoint
        coordinates in the active coordinate system (RR, Rθ, RZ
        for cylindrical; RR, Rθ, RΦ for spherical).  Note that the
        Rθ and RΦ scale factors are interpreted as angular
        offsets.  For example, if CSYS = 1, RX, RY, RZ input of
        (1.5,10,3) would scale the specified keypoints 1.5 times
        in the radial and 3 times in the Z direction, while adding
        an offset of 10 degrees to the keypoints.  Zero, blank, or
        negative scale factor values are assumed to be 1.0.  Zero
        or blank angular offsets have no effect.

    Returns
    -------
    str
        MAPDL command output.

    Examples
    --------
    Create a basic cylinder by extruding a circle.

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 0, 0, 1)
    >>> k2 = mapdl.k("", 0, 0, 0.5)
    >>> carc0 = mapdl.circle(k0, 1, k1)
    >>> a0 = mapdl.al(*carc0)
    >>> mapdl.vext(a0, dz=4)

    Create a tapered cylinder.

    >>> mapdl.vdele('all')
    >>> mapdl.vext(a0, dz=4, rx=0.3, ry=0.3, rz=1)

    Notes
    -----
    Generates additional volumes (and their corresponding
    keypoints, lines, and areas) by extruding and scaling a
    pattern of areas in the active coordinate system.

    If element attributes have been associated with the input area
    via the AATT command, the opposite area generated by the VEXT
    operation will also have those attributes (i.e., the element
    attributes from the input area are copied to the opposite
    area).  Note that only the area opposite the input area will
    have the same attributes as the input area; the areas adjacent
    to the input area will not.

    If the areas are meshed or belong to meshed volumes, a 3-D
    mesh can be extruded with this command.  Note that the NDIV
    argument on the ESIZE command should be set before extruding
    the meshed areas.

    Scaling of the input areas, if specified, is performed first,
    followed by the extrusion.

    In a non-Cartesian coordinate system, the VEXT command locates
    the end face of the volume based on the active coordinate
    system.  However, the extrusion is made along a straight line
    between the end faces.  Note that solid modeling in a toroidal
    coordinate system is not recommended.

    Caution:: : Use of the VEXT command can produce unexpected
    results when operating in a non-Cartesian coordinate system.
    For a detailed description of the possible problems that may
    occur, see Solid Modeling in the Modeling and Meshing Guide.
    """
    command = f"VEXT,{na1},{na2},{ninc},{dx},{dy},{dz},{rx},{ry},{rz}"
    return self.run(command, **kwargs)


def vgen(self, itime="", nv1="", nv2="", ninc="", dx="", dy="", dz="",
         kinc="", noelem="", imove="", **kwargs):
    """Generates additional volumes from a pattern of volumes.

    APDL Command: VGEN

    Parameters
    ----------
    itime
        Do this generation operation a total of ITIMEs, incrementing all
        keypoints in the given pattern automatically (or by KINC) each time
        after the first.  ITIME must be > 1 for generation to occur.

    nv1, nv2, ninc
        Generate volumes from pattern beginning with NV1 to NV2 (defaults
        to NV1) in steps of NINC (defaults to 1).  If NV1 = ALL, NV2 and
        NINC are ignored and the pattern is all selected volumes [VSEL].
        If NV1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NV1 (NV2 and NINC are ignored).

    dx, dy, dz
        Keypoint location increments in the active coordinate system (--,
        Dθ, DZ for cylindrical, --, Dθ, -- for spherical).

    kinc
        Keypoint increment between generated sets.  If zero, the lowest
        available keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies if elements and nodes are also to be generated:

        0 - Generate nodes and elements associated with the original volumes, if they
            exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether to redefine the existing volumes:

        0 - Generate additional volumes as requested with the ITIME argument.

        1 - Move original volumes to new position retaining the same keypoint line, and
            area numbers (ITIME, KINC, and NOELEM are ignored).
            Corresponding meshed items are also moved if not needed at
            their original position.

    Notes
    -----
    Generates additional volumes (and their corresponding keypoints, lines,
    areas and mesh) from a given volume pattern.  The MAT, TYPE, REAL, and
    ESYS attributes are based upon the volumes in the pattern and not upon
    the current settings of the pointers.  End slopes of the generated
    lines remain the same (in the active coordinate system) as those of the
    given pattern.  For example, radial slopes remain radial, etc.
    Generations which produce volumes of a size or shape different from the
    pattern (i.e., radial generations in cylindrical systems, radial and
    phi generations in spherical systems, and theta generations in
    elliptical systems) are not allowed.  Note that solid modeling in a
    toroidal coordinate system is not recommended.  Volume, area, and line
    numbers are automatically assigned (beginning with the lowest available
    values [NUMSTR]).
    """
    command = f"VGEN,{itime},{nv1},{nv2},{ninc},{dx},{dy},{dz},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def vlist(self, nv1="", nv2="", ninc="", **kwargs):
    """Lists the defined volumes.

    APDL Command: VLIST

    Parameters
    ----------
    nv1, nv2, ninc
        List volumes from NV1 to NV2 (defaults to NV1) in steps of NINC
        (defaults to 1).  If NV1 = ALL (default), NV2 and NINC are ignored
        and all selected volumes [VSEL] are listed.  If NV1 = P, graphical
        picking is enabled and all remaining command fields are ignored
        (valid only in the GUI).  A component name may also be substituted
        for NV1 (NV2 and NINC are ignored).

    Notes
    -----
    An attribute (TYPE, MAT, REAL, or ESYS) listed as a zero is unassigned;
    one listed as a positive value indicates that the attribute was
    assigned with the VATT command (and will not be reset to zero if the
    mesh is cleared); one listed as a negative value indicates that the
    attribute was assigned using the attribute pointer [TYPE, MAT, REAL, or
    ESYS] that was active during meshing (and will be reset to zero if the
    mesh is cleared).  A "-1" in the "nodes" column indicates that the
    volume has been meshed but there are no interior nodes.  The volume
    size is listed only if a VSUM command has been performed on the volume.
    Volume orientation attributes (KZ1 and KZ2) are listed only if a
    VEORIENT command was previously used to define an orientation for the
    volume.

    This command is valid in any processor.
    """
    command = f"VLIST,{nv1},{nv2},{ninc}"
    return self.run(command, **kwargs)


def vlscale(self, nv1="", nv2="", ninc="", rx="", ry="", rz="", kinc="",
            noelem="", imove="", **kwargs):
    """Generates a scaled set of volumes from a pattern of volumes.

    APDL Command: VLSCALE

    Parameters
    ----------
    nv1, nv2, ninc
        Set of volumes (NV1 to NV2 in steps of NINC) that defines the
        pattern to be scaled.  NV2 defaults to NV1, NINC defaults to 1.  If
        NV1 = ALL, NV2 and NINC are ignored and the pattern is defined by
        all selected volumes.  If NV1 = P, graphical picking is enabled and
        all remaining command fields are ignored (valid only in the GUI).
        A component name may also be substituted for NV1 (NV2 and NINC are
        ignored).

    rx, ry, rz
        Scale factors to be applied to the X, Y, and Z keypoint coordinates
        in active coordinate system (RR, Rθ, RZ for cylindrical; RR, Rθ, RΦ
        for spherical).  Note that the Rθ and RΦ scale factors are
        interpreted as angular offsets.  For example, if CSYS = 1, RX, RY,
        RZ input of (1.5,10,3) would scale the specified keypoints 1.5
        times in the radial and 3 times in the Z direction, while adding an
        offset of 10 degrees to the keypoints.  Zero, blank, or negative
        scale factor values are assumed to be 1.0.  Zero or blank angular
        offsets have no effect.

    kinc
        Increment to be applied to keypoint numbers for generated set.  If
        zero, the lowest available keypoint numbers will be assigned
        [NUMSTR].

    noelem
        Specifies whether nodes and elements are also to be generated:

        0 - Nodes and elements associated with the original volumes will be generated
            (scaled) if they exist.

        1 - Nodes and elements will not be generated.

    imove
        Specifies whether volumes will be moved or newly defined:

        0 - Additional volumes will be generated.

        1 - Original volumes will be moved to new position (KINC and NOELEM are ignored).
            Use only if the old volumes are no longer needed at their
            original positions.  Corresponding meshed items are also moved
            if not needed at their original position.

    Notes
    -----
    Generates a scaled set of volumes (and their corresponding keypoints,
    lines, areas,  and mesh) from a pattern of volumes.  The MAT, TYPE,
    REAL, and ESYS attributes are based on the volumes in the pattern and
    not the current settings.  Scaling is done in the active coordinate
    system.  Volumes in the pattern could have been generated in any
    coordinate system.  However, solid modeling in a toroidal coordinate
    system is not recommended.
    """
    command = f"VLSCALE,{nv1},{nv2},{ninc},{rx},{ry},{rz},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def voffst(self, narea="", dist="", kinc="", **kwargs):
    """Generates a volume, offset from a given area.

    APDL Command: VOFFST

    Parameters
    ----------
    narea
        Area from which generated volume is to be offset.  If NAREA = P,
        graphical picking is enabled and all remaining command fields are
        ignored (valid only in the GUI).

    dist
        Distance normal to given area at which keypoints for generated
        volume are to be located.  Positive normal is determined from the
        right-hand rule keypoint order.

    kinc
        Increment to be applied to the keypoint numbers between sets.  If
        zero, keypoint numbers will be automatically assigned beginning
        with the lowest available value [NUMSTR].

    Notes
    -----
    Generates a volume (and its corresponding keypoints, lines, and areas)
    by offsetting from an area.  The direction of the offset varies with
    the given area normal.  End slopes of the generated lines remain the
    same as those of the given pattern.

    If element attributes have been associated with the input area via the
    AATT command, the opposite area generated by the VOFFST operation will
    also have those attributes (i.e., the element attributes from the input
    area are copied to the opposite area).  Note that only the area
    opposite the input area will have the same attributes as the input
    area; the areas adjacent to the input area will not.

    If the areas are meshed or belong to meshed volumes, a 3-D mesh can be
    extruded with this command.  Note that the NDIV argument on the ESIZE
    command should be set before extruding the meshed areas.
    """
    command = f"VOFFST,{narea},{dist},{kinc}"
    return self.run(command, **kwargs)


def vplot(self, nv1="", nv2="", ninc="", degen="", scale="", **kwargs):
    """Displays the selected volumes.

    APDL Command: VPLOT

    Parameters
    ----------
    nv1, nv2, ninc
        Display volumes from NV1 to NV2 (defaults to NV1) in steps of NINC
        (defaults to 1).  If NV1 = ALL (default), NV2 and NINC are ignored
        and all selected volumes [VSEL] are displayed.

    degen
        Degeneracy marker:

        (blank) - No degeneracy marker is used (default).

        DEGE - A red star is placed on keypoints at degeneracies (see the Modeling and Meshing
               Guide).  Not available if /FACET,WIRE is set.

    scale
        Scale factor for the size of the degeneracy-marker star.  The scale
        is the size in window space (-1 to 1 in both directions) (defaults
        to .075).

    Notes
    -----
    Displays selected volumes.  (Only volumes having areas within the
    selected area set [ASEL] will be plotted.)  With PowerGraphics on
    [/GRAPHICS,POWER], VPLOT will display only the currently selected
    areas. This command is also a utility command, valid anywhere.  The
    degree of tessellation used to plot the volumes is set through the
    /FACET command.
    """
    command = f"VPLOT,{nv1},{nv2},{ninc},{degen},{scale}"
    return self.run(command, **kwargs)


def vrotat(self, na1="", na2="", na3="", na4="", na5="",
           na6="", pax1="", pax2="", arc="", nseg="",
           **kwargs) -> str:
    """Generate cylindrical volumes by rotating an area pattern about

    APDL Command: VROTAT
    an axis.

    Parameters
    ----------
    na1, na2, na3, . . . , na6
        List of areas in the pattern to be rotated (6 maximum if
        using keyboard entry).  Areas must lie to one side of, and
        in the plane of, the axis of rotation.  If NA1 = ALL,
        all selected areas will define the pattern to be rotated.
        A component name may also be substituted for NA1.

    pax1, pax2
        Keypoints defining the axis about which the area pattern
        is to be rotated.

    arc
        Arc length (in degrees).  Positive follows right-hand rule
        about PAX1-PAX2 vector.  Defaults to 360.

    nseg
        Number of volumes (8 maximum) around circumference.
        Defaults to minimum required for 90 degrees (maximum) arcs, i.e.,
        4 for 360 degrees, 3 for 270 degrees, etc.

    Returns
    -------
    str
        MAPDL command output.

    Examples
    --------
    Rotate a circle about the Z axis to create a hoop.

    First, create a circle offset from origin on the XZ plane.

    >>> hoop_radius = 10
    >>> hoop_thickness = 0.5
    >>> k0 = mapdl.k("", hoop_radius, 0, 0)
    >>> k1 = mapdl.k("", hoop_radius, 1, 0)
    >>> k2 = mapdl.k("", hoop_radius, 0, hoop_thickness)
    >>> carc0 = mapdl.circle(k0, 1, k1)
    >>> a0 = mapdl.al(*carc0)

    Create a hoop by rotating it about an axis defined by two
    keypoints.

    >>> k_axis0 = mapdl.k("", 0, 0, 0)
    >>> k_axis1 = mapdl.k("", 0, 0, 1)
    mapdl.vrotat(a0, pax1=k_axis0, pax2=k_axis1)

    Notes
    -----
    Generates cylindrical volumes (and their corresponding
    keypoints, lines, and areas) by rotating an area pattern (and
    its associated line and keypoint patterns) about an axis.
    Keypoint patterns are generated at regular angular locations
    (based on a maximum spacing of 90 degrees).  Line patterns are
    generated at the keypoint patterns.  Arc lines are also
    generated to connect the keypoints circumferentially.
    Keypoint, line, area, and volume numbers are automatically
    assigned (beginning with the lowest available values).
    Adjacent lines use a common keypoint, adjacent areas use a
    common line, and adjacent volumes use a common area.

    To generate a single volume with an arc greater than 180 degrees,
    NSEG must be greater than or equal to 2.

    If element attributes have been associated with the input area
    via the AATT command, the opposite area generated by the
    VROTAT operation will also have those attributes (i.e., the
    element attributes from the input area are copied to the
    opposite area).  Note that only the area opposite the input
    area will have the same attributes as the input area; the
    areas adjacent to the input area will not.

    If the given areas are meshed or belong to meshed volumes, the
    2-D mesh can be rotated (extruded) to a 3-D mesh. See the
    Modeling and Meshing Guide for more information.  Note that
    the NDIV argument on the ESIZE command should be set before
    extruding the meshed areas.
    """
    command = f"VROTAT,{na1},{na2},{na3},{na4},{na5},{na6},{pax1},{pax2},{arc},{nseg}"
    return self.run(command, **kwargs)


def vsum(self, lab="", **kwargs):
    """Calculates and prints geometry statistics of the selected volumes.

    APDL Command: VSUM

    Parameters
    ----------
    lab
        Controls the degree of tessellation used in the calculation of area
        properties.  If LAB = DEFAULT, area calculations will use the
        degree of tessellation set through the /FACET command.  If LAB =
        FINE, area calculations are based on a finer tessellation.

    Notes
    -----
    Calculates and prints geometry statistics (volume, centroid location,
    moments of inertia, etc.) associated with the selected volumes.
    Geometry items are reported in the global Cartesian coordinate system.
    A unit density is assumed unless the volumes have a material
    association via the VATT command.  Items calculated by VSUM and later
    retrieved by a *GET or *VGET command are valid only if the model is not
    modified after the VSUM command is issued.

    Setting a finer degree of tessellation will provide area calculations
    with greater accuracy, especially for thin, hollow models.  However,
    using a finer degree of tessellation requires longer processing.

    For very thin volumes, such that the ratio of the minimum to the
    maximum dimension is less than 0.01, the VSUM command can provide
    erroneous volume information.  To ensure that such calculations are
    accurate, make certain that you subdivide such volumes so that the
    ratio of the minimum to the maximum is at least 0.05.
    """
    command = f"VSUM,{lab}"
    return self.run(command, **kwargs)


def vsymm(self, ncomp="", nv1="", nv2="", ninc="", kinc="", noelem="",
          imove="", **kwargs) -> str:
    """Generate volumes from a volume pattern by symmetry reflection.

    APDL Command: VSYMM

    Parameters
    ----------
    ncomp
        Symmetry key:

        - ``'X'`` : X symmetry (default).
        - ``'Y'`` : Y symmetry.
        - ``'Z'`` : Z symmetry.

    nv1, nv2, ninc
        Reflect volumes from pattern beginning with NV1 to NV2
        (defaults to NV1) in steps of NINC (defaults to 1).  If
        NV1 = ALL, NV2 and NINC are ignored and the pattern is all
        selected volumes [VSEL].  A component name may also be
        substituted for NV1 (NV2 and NINC are ignored).

    kinc
        Keypoint increment between sets.  If zero, the lowest
        available keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies whether nodes and elements are also to be generated:

        0 - Generate nodes and elements associated with the
            original volumes, if they exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether volumes will be moved or newly defined:

        0 - Generate additional volumes.

        1 - Move original volumes to new position retaining the
            same keypoint numbers (KINC and NOELEM are ignored).
            Corresponding meshed items are also moved if not
            needed at their original position.

    Returns
    -------
    str
        MAPDL command output.

    Examples
    --------
    Create four blocks by reflecting a single block across the X
    component and then the Y component.

    >>> vnum = mapdl.blc4(1, 1, 1, 1, depth=1)
    >>> mapdl.vsymm('X', vnum)
    >>> output = mapdl.vsymm('Y', 'ALL')
    >>> print(output)
    SYMMETRY TRANSFORMATION OF VOLUMES       USING COMPONENT  Y
       SET IS ALL SELECTED VOLUMES

    Notes
    -----
    Generates a reflected set of volumes (and their corresponding
    keypoints, lines, areas and mesh) from a given volume pattern
    by a symmetry reflection (see analogous node symmetry command,
    NSYM).  The MAT, TYPE, REAL, and ESYS attributes are based
    upon the volumes in the pattern and not upon the current
    settings.  Reflection is done in the active coordinate system
    by changing a particular coordinate sign.  The active
    coordinate system must be a Cartesian system.  Volumes in the
    pattern may have been generated in any coordinate system.
    However, solid modeling in a toroidal coordinate system is not
    recommended.  Volumes are generated as described in the VGEN
    command.

    See the ESYM command for additional information about symmetry
    elements.
    """
    command = f"VSYMM,{ncomp},{nv1},{nv2},{ninc},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def vtran(self, kcnto="", nv1="", nv2="", ninc="", kinc="", noelem="",
          imove="", **kwargs):
    """Transfers a pattern of volumes to another coordinate system.

    APDL Command: VTRAN

    Parameters
    ----------
    kcnto
        Reference number of coordinate system where the pattern is to be
        transferred.  Transfer occurs from the active coordinate system.
        The coordinate system type and parameters of KCNTO must be the same
        as the active system.

    nv1, nv2, ninc
        Transfer volumes from pattern beginning with NV1 to NV2 (defaults
        to NV1) in steps of NINC (defaults to 1).  If NV1 = ALL, NV2 and
        NINC are ignored and the pattern is all selected volumes [VSEL].
        If NV1 = P, graphical picking is enabled and all remaining command
        fields are ignored (valid only in the GUI).  A component name may
        also be substituted for NV1 (NV2 and NINC are ignored).

    kinc
        Keypoint increment between sets.  If zero, the lowest available
        keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies whether elements and nodes are also to be generated:

        0 - Generate nodes and elements associated with the original volumes, if they
            exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether to redefine the existing volumes:

        0 - Generate additional volumes.

        1 - Move original volumes to new position retaining the same keypoint numbers (KINC
            and NOELEM are ignored).  Corresponding meshed items are also
            moved if not needed at their original position.

    Notes
    -----
    Transfers a pattern of volumes (and their corresponding keypoints,
    lines, areas and mesh) from one coordinate system to another (see
    analogous node transfer command, TRANSFER).  The MAT, TYPE, REAL, and
    ESYS attributes are based upon the volumes in the pattern and not upon
    the current settings.  Coordinate systems may be translated and rotated
    relative to each other.  Initial pattern may be generated in any
    coordinate system.  However, solid modeling in a toroidal coordinate
    system is not recommended.  Coordinate and slope values are interpreted
    in the active coordinate system and are transferred directly.  Volumes
    are generated as described in the VGEN command.
    """
    command = f"VTRAN,{kcnto},{nv1},{nv2},{ninc},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)



from ansys.mapdl.core._commands import parse


def a(self, p1="", p2="", p3="", p4="", p5="", p6="", p7="", p8="", p9="",
      p10="", p11="", p12="", p13="", p14="", p15="", p16="", p17="",
      p18="", **kwargs) -> int:
    """Define an area by connecting keypoints.

    APDL Command: A

    Parameters
    ----------
    p1, p2, p3, . . . , p18
        List of keypoints defining the area (18 maximum if using
        keyboard entry).  At least 3 keypoints must be entered.

    Returns
    -------
    int
        The area number of the generated area.

    Examples
    --------
    Create a simple triangle in the XY plane using three keypoints.

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 1, 0, 0)
    >>> k2 = mapdl.k("", 0, 1, 0)
    >>> a0 = mapdl.a(k0, k1, k2)
    >>> a0
    1

    Notes
    -----
    Keypoints (P1 through P18) must be input in a clockwise or
    counterclockwise order around the area.  This order also
    determines the positive normal direction of the area according
    to the right-hand rule.  Existing lines between adjacent
    keypoints will be used; missing lines are generated "straight"
    in the active coordinate system and assigned the lowest
    available numbers [NUMSTR].  If more than one line exists
    between two keypoints, the shorter one will be chosen.  If the
    area is to be defined with more than four keypoints, the
    required keypoints and lines must lie on a constant coordinate
    value in the active coordinate system (such as a plane or a
    cylinder).  Areas may be redefined only if not yet attached to
    a volume.  Solid modeling in a toroidal coordinate system is
    not recommended.
    """
    command = f"A,{p1},{p2},{p3},{p4},{p5},{p6},{p7},{p8},{p9},{p10},{p11},{p12},{p13},{p14},{p15},{p16},{p17},{p18}"
    return parse.parse_a(self.run(command, **kwargs))


def aatt(self, mat="", real="", type_="", esys="", secn="", **kwargs):
    """Associates element attributes with the selected, unmeshed areas.

    APDL Command: AATT

    Parameters
    ----------
    mat
        The material number to be associated with selected, unmeshed areas.

    real
        The real constant set number to be associated with selected,
        unmeshed areas.

    type\_
        The type number to be associated with selected, unmeshed areas.

    esys
        The coordinate system number to be associated with selected,
        unmeshed areas.

    secn
        The section number to be associated with selected unmeshed areas.

    Notes
    -----
    Areas subsequently generated from the areas will also have these
    attributes.  These element attributes will be used when the areas are
    meshed.  If an area does not have attributes associated with it (by
    this command) at the time it is meshed, the attributes are obtained
    from the then current MAT, REAL, TYPE, ESYS, and SECNUM command
    settings.  Reissue the AATT command (before areas are meshed) to change
    the attributes.  A zero (or blank) argument removes the corresponding
    association. If any of the arguments MAT, REAL, TYPE, ESYS, or SECN are
    defined as -1, then that value will be left unchanged in the selected
    set.

    In some cases, ANSYS can proceed with an area meshing operation even
    when no logical element type has been assigned via AATT,,,TYPE or TYPE.
    For more information, see the discussion on setting element attributes
    in Meshing Your Solid Model in the Modeling and Meshing Guide.
    """
    command = f"AATT,{mat},{real},{type_},{esys},{secn}"
    return self.run(command, **kwargs)


def adele(self, na1="", na2="", ninc="", kswp="", **kwargs):
    """Deletes unmeshed areas.

    APDL Command: ADELE

    Parameters
    ----------
    na1, na2, ninc
        Delete areas from NA1 to NA2 (defaults to NA1) in steps of NINC
        (defaults to 1).  If NA1 = ALL, NA2 and NINC are ignored and all
        selected areas [ASEL] are deleted.  If NA1 = P, graphical picking
        is enabled and all remaining arguments are ignored (valid only in
        the GUI).  A component name may also be substituted for NA1 (NA2
        and NINC are ignored).

    kswp
        Specifies whether keypoints and lines are also to be deleted:

        0 - Delete areas only (default).

        1 - Delete areas, as well as keypoints and lines attached to specified areas but
            not shared by other areas.

    Notes
    -----
    An area attached to a volume cannot be deleted unless the volume is
    first deleted.
    """
    command = f"ADELE,{na1},{na2},{ninc},{kswp}"
    return self.run(command, **kwargs)


def adgl(self, na1="", na2="", ninc="", **kwargs):
    """Lists keypoints of an area that lie on a parametric degeneracy.

    APDL Command: ADGL

    Parameters
    ----------
    na1, na2, ninc
        List keypoints that lie on a parametric degeneracy on areas from
        NA1 to NA2 (defaults to NA1) in steps of NINC (defaults to 1).  If
        NA1 = ALL (default), NA2 and NINC will be ignored and keypoints on
        all selected areas [ASEL] will be listed.  If NA1 = P, graphical
        picking is enabled and all remaining arguments are ignored (valid
        only in the GUI).  A component name may be substituted in NA1 (NA2
        and NINC will be ignored).

    Notes
    -----
    See the Modeling and Meshing Guide for details on parametric
    degeneracies.

    This command is valid in any processor.
    """
    command = f"ADGL,{na1},{na2},{ninc}"
    return self.run(command, **kwargs)


def adrag(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nlp1="",
          nlp2="", nlp3="", nlp4="", nlp5="", nlp6="", **kwargs) -> str:
    """Generate areas by dragging a line pattern along a path.

    APDL Command: ADRAG

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl6
        List of lines in the pattern to be dragged (6 maximum if
        using keyboard entry).  Lines should form a continuous
        pattern (no more than two lines connected to any one
        keypoint.  If NL1 = ALL, all selected lines (except those
        that define the drag path) will be swept along the path.
        A component name may also be substituted for NL1.

    nlp1, nlp2, nlp3, . . . , nlp6
        List of lines defining the path along which the pattern is
        to be dragged (6 maximum if using keyboard entry).  Must
        be a continuous set of lines.

    Returns
    -------
    str
        MAPDL command output.

    Examples
    --------
    Drag a circle between two keypoints to create an area

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 0, 0, 1)
    >>> carc = mapdl.circle(k0, 1, k1, arc=90)
    >>> l0 = mapdl.l(k0, k1)
    >>> output = mapdl.adrag(carc[0], nlp1=l0)
    >>> print(output)
    DRAG LINES:
         1,
    ALONG LINES
         2,

    Notes
    -----
    Generates areas (and their corresponding keypoints and lines)
    by sweeping a given line pattern along a characteristic drag
    path.  If the drag path consists of multiple lines, the drag
    direction is determined by the sequence in which the path
    lines are input (NLP1, NLP2, etc.).  If the drag path is a
    single line (NLP1), the drag direction is from the keypoint on
    the drag line that is closest to the first keypoint of the
    given line pattern to the other end of the drag line.

    The magnitude of the vector between the keypoints of the given
    pattern and the first path keypoint remains constant for all
    generated keypoint patterns and the path keypoints.  The
    direction of the vector relative to the path slope also
    remains constant so that patterns may be swept around curves.

    Keypoint, line, and area numbers are automatically assigned
    (beginning with the lowest available values [NUMSTR]).
    Adjacent lines use a common keypoint.  Adjacent areas use a
    common line.  For best results, the entities to be dragged
    should be orthogonal to the start of the drag path.  Drag
    operations that produce an error message may create some of
    the desired entities prior to terminating.
    """
    command = f"ADRAG,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nlp1},{nlp2},{nlp3},{nlp4},{nlp5},{nlp6}"
    return self.run(command, **kwargs)


def afillt(self, na1="", na2="", rad="", **kwargs):
    """Generates a fillet at the intersection of two areas.

    APDL Command: AFILLT

    Parameters
    ----------
    na1
        Number of the first intersecting area.  If NA1 = P, graphical
        picking is enabled and all remaining arguments are ignored (valid
        only in the GUI).

    na2
        Number of the second intersecting area.

    rad
        Radius of fillet to be generated.

    Notes
    -----
    Generates an area of constant fillet radius at the intersection of two
    areas using a series of Boolean operations.  Corresponding lines and
    keypoints are also generated.  See BOPTN command for an explanation of
    the options available to Boolean operations.  If areas do not initially
    intersect at a common line, use the AINA command.
    """
    command = f"AFILLT,{na1},{na2},{rad}"
    return self.run(command, **kwargs)


def agen(self, itime="", na1="", na2="", ninc="", dx="", dy="", dz="",
         kinc="", noelem="", imove="", **kwargs):
    """Generates additional areas from a pattern of areas.

    APDL Command: AGEN

    Parameters
    ----------
    itime
        Do this generation operation a total of ITIMEs, incrementing all
        keypoints in the given pattern automatically (or by KINC) each time
        after the first.  ITIME must be more than 1 for generation to
        occur.

    na1, na2, ninc
        Generate areas from the pattern of areas NA1 to NA2 (defaults to
        NA1) in steps of NINC (defaults to 1).  If NA1 = ALL, NA2 and NINC
        are ignored and the pattern is all selected areas [ASEL].  If NA1 =
        P, graphical picking is enabled and all remaining arguments are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NA1 (NA2 and NINC are ignored).

    dx, dy, dz
        Keypoint location increments in the active coordinate system (--,
        D θ, DZ for cylindrical;  --, D θ, -- for spherical).

    kinc
        Keypoint number increment between generated sets.  If zero, the
        lowest available keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies if elements and nodes are also to be generated:

        0 - Generate nodes and elements associated with the original areas, if they exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether to redefine the existing areas:

        0 - Generate new areas as requested with the ITIME argument.

        1 - Move original areas to new position, retaining the same keypoint numbers
            (ITIME, KINC, and NOELEM are ignored).  If the original areas
            are needed in the original position (e.g.,  they may be
            attached to a volume), they are not moved, and new areas are
            generated instead.  Meshed items corresponding to moved areas
            are also moved if not needed at their original position.

    Notes
    -----
    Generates additional areas (and their corresponding keypoints, lines
    and mesh) from a given area pattern.  The MAT, TYPE, REAL, ESYS, and
    SECNUM attributes of the new areas are based upon the areas in the
    pattern and not upon the current settings of the pointers.  End slopes
    of the generated lines remain the same (in the active coordinate
    system) as those of the given pattern.  For example, radial slopes
    remain radial.  Generations which produce areas of a size or shape
    different from the pattern (i.e., radial generations in cylindrical
    systems, radial and phi generations in spherical systems, and theta
    generations in elliptical systems) are not allowed.  Solid modeling in
    a toroidal coordinate system is not recommended.  Area and line numbers
    are automatically assigned, beginning with the lowest available values
    [NUMSTR].
    """
    command = f"AGEN,{itime},{na1},{na2},{ninc},{dx},{dy},{dz},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def al(self, l1="", l2="", l3="", l4="", l5="", l6="", l7="", l8="", l9="",
       l10="", **kwargs) -> int:
    """Generate an area bounded by previously defined lines.

    APDL Command: AL

    Parameters
    ----------
    l1, l2, l3, . . . , l10
        List of lines defining area.  The minimum number of lines
        is 3.  The positive normal of the area is controlled by
        the direction of L1 using the right-hand rule.  A negative
        value of L1 reverses the normal direction.  If L1 = ALL,
        use all selected lines with L2 defining the normal (L3 to
        L10 are ignored and L2 defaults to the lowest numbered
        selected line).  A component name may also be substituted
        for L1.

    Returns
    -------
    int
        Area number of the generated area.

    Examples
    --------
    Create an area from four lines

    >>> k0 = mapdl.k("", 0, 0, 0)
    >>> k1 = mapdl.k("", 1, 0, 0)
    >>> k2 = mapdl.k("", 1, 1, 0)
    >>> k3 = mapdl.k("", 0, 1, 0)
    >>> l0 = mapdl.l(k0, k1)
    >>> l1 = mapdl.l(k1, k2)
    >>> l2 = mapdl.l(k2, k3)
    >>> l3 = mapdl.l(k3, k0)
    >>> anum = mapdl.al(l0, l1, l2, l3)
    >>> anum
    1

    Notes
    -----
    Lines may be input (once each) in any order and must form a
    simply connected closed curve.  If the area is defined with
    more than four lines, the lines must also lie in the same
    plane or on a constant coordinate value in the active
    coordinate system (such as a plane or a cylinder).

    Solid modeling in a toroidal coordinate system is not
    recommended.  Areas may be redefined only if not yet attached
    to a volume.

    This command is valid in any processor.
    """
    command = f"AL,{l1},{l2},{l3},{l4},{l5},{l6},{l7},{l8},{l9},{l10}"
    return parse.parse_a(self.run(command, **kwargs))


def alist(self, na1="", na2="", ninc="", lab="", **kwargs):
    """Lists the defined areas.

    APDL Command: ALIST

    Parameters
    ----------
    na1, na2, ninc
        List areas from NA1 to NA2 (defaults to NA1) in steps of NINC
        (defaults to 1).  If NA1 = ALL (default), NA2 and NINC are ignored
        and all selected areas [ASEL] are listed.  If NA1 = P, graphical
        picking is enabled and all remaining arguments are ignored (valid
        only in the GUI).  A component name may also be substituted for NA1
        (NA2 and NINC are ignored).

    lab
        Determines what type of listing is used (one of the following):

        (blank) - Prints information about all areas in the specified range.

        HPT - Prints information about only those areas that contain hard points.

    Notes
    -----
    An attribute (TYPE, MAT, REAL, or ESYS) listed as a zero is unassigned;
    one listed as a positive value indicates that the attribute was
    assigned with the AATT command (and will not be reset to zero if the
    mesh is cleared); one listed as a negative value indicates that the
    attribute was assigned using the attribute pointer [TYPE, MAT, REAL, or
    ESYS] that was active during meshing (and will be reset to zero if the
    mesh is cleared).  A "-1" in the "nodes" column indicates that the area
    has been meshed but there are no interior nodes.  The area size is
    listed only if an ASUM command has been performed on the area.
    """
    command = f"ALIST,{na1},{na2},{ninc},{lab}"
    return self.run(command, **kwargs)


def anorm(self, anum="", noeflip="", **kwargs):
    """Reorients area normals.

    APDL Command: ANORM

    Parameters
    ----------
    anum
        Area number having the normal direction that the reoriented areas
        are to match.

    noeflip
        Indicates whether you want to change the normal direction of the
        existing elements on the reoriented area(s) so that they are
        consistent with each area's new normal direction.

        0 - Make the normal direction of existing elements on the reoriented area(s)
            consistent with each area's new normal direction (default).

        1 - Do not change the normal direction of existing elements on the reoriented
            area(s).

    Notes
    -----
    Reorients areas so that their normals are consistent with that of a
    specified area.

    If any of the areas have inner loops, the ANORM command will consider
    the inner loops when it reorients the area normals.

    You cannot use the ANORM command to change the normal direction of any
    element that has a body or surface load.  We recommend that you apply
    all of your loads only after ensuring that the element normal
    directions are acceptable.

    Real constants (such as nonuniform shell thickness and tapered beam
    constants) may be invalidated by an element reversal.

    See Revising Your Model of the Modeling and Meshing Guide for more
    information.
    """
    command = f"ANORM,{anum},{noeflip}"
    return self.run(command, **kwargs)


def aoffst(self, narea="", dist="", kinc="", **kwargs):
    """Generates an area, offset from a given area.

    APDL Command: AOFFST

    Parameters
    ----------
    narea
        Area from which generated area is to be offset.  If NAREA = ALL,
        offset from all selected areas [ASEL].  If NAREA = P, graphical
        picking is enabled and all remaining arguments are ignored (valid
        only in the GUI).

    dist
        Distance normal to given area at which keypoints for generated area
        are to be located.  Positive normal is determined from the right-
        hand-rule keypoint order.

    kinc
        Keypoint increment between areas.  If zero, the lowest available
        keypoint numbers are assigned [NUMSTR].

    Notes
    -----
    Generates an area (and its corresponding keypoints and lines) offset
    from a given area.  The direction of the offset varies with the given
    area normal.  End slopes of the generated lines remain the same as
    those of the given pattern.  Area and line numbers are automatically
    assigned, beginning with the lowest available values [NUMSTR].
    """
    command = f"AOFFST,{narea},{dist},{kinc}"
    return self.run(command, **kwargs)


def aplot(self, na1="", na2="", ninc="", degen="", scale="", **kwargs):
    """Displays the selected areas.

    APDL Command: APLOT

    Parameters
    ----------
    na1, na2, ninc
        Displays areas from NA1 to NA2 (defaults to NA1) in steps of NINC
        (defaults to 1).  If NA1 = ALL (default), NA2 and NINC are ignored
        and all selected areas [ASEL] are displayed.

    degen
        Degeneracy marker:

        (blank) - No degeneracy marker is used (default).

        DEGE - A red star is placed on keypoints at degeneracies (see the Modeling and Meshing
               Guide ).  Not available if /FACET,WIRE is set.

    scale
        Scale factor for the size of the degeneracy-marker star.  The scale
        is the size in window space (-1 to 1 in both directions) (defaults
        to .075).

    Notes
    -----
    This command is valid in any processor.  The degree of tessellation
    used to plot the selected areas is set through the /FACET command.
    """
    command = f"APLOT,{na1},{na2},{ninc},{degen},{scale}"
    return self.run(command, **kwargs)


def areverse(self, anum="", noeflip="", **kwargs):
    """Reverses the normal of an area, regardless of its connectivity or mesh

    APDL Command: AREVERSE
    status.

    Parameters
    ----------
    anum
        Area number of the area whose normal is to be reversed.  If ANUM =
        ALL, the normals of all selected areas will be reversed.  If ANUM =
        P, graphical picking is enabled.  A component name may also be
        substituted for ANUM.

    noeflip
        Indicates whether you want to change the normal direction of the
        existing elements on the reversed area(s) so that they are
        consistent with each area's new normal direction.

        0 - Make the normal direction of existing elements on the reversed area(s)
            consistent with each area's new normal direction (default).

        1 - Do not change the normal direction of existing elements on the reversed
            area(s).

    Notes
    -----
    You cannot use the AREVERSE command to change the normal direction of
    any element that has a body or surface load.  We recommend that you
    apply all of your loads only after ensuring that the element normal
    directions are acceptable. Also, you cannot use this command to change
    the normal direction for areas attached to volumes because IGES and ANF
    data is unchanged by reversal. Reversed areas that are attached to
    volumes need to be reversed again when imported.

    Real constants (such as nonuniform shell thickness and tapered beam
    constants) may be invalidated by an element reversal.

    See Revising Your Model in the Modeling and Meshing Guide for more
    information.
    """
    command = f"AREVERSE,{anum},{noeflip}"
    return self.run(command, **kwargs)


def arotat(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", pax1="",
           pax2="", arc="", nseg="", **kwargs):
    """Generates cylindrical areas by rotating a line pattern about an axis.

    APDL Command: AROTAT

    Parameters
    ----------
    nl1, nl2, nl3, . . . , nl6
        List of lines in the pattern to be rotated (6 maximum if using
        keyboard entry of NL1 to NL6).  The lines must lie in the plane of
        the axis of rotation.  If NL1 = P, graphical picking is enabled and
        all remaining arguments are ignored (valid only in the GUI).  If
        NL1 = ALL, all selected lines will define the pattern to be
        rotated.  A component name may also be substituted for NL1.

    pax1, pax2
        Keypoints defining the axis about which the line pattern is to be
        rotated.

    arc
        Arc length (in degrees).  Positive follows right-hand rule about
        PAX1-PAX2 vector.  Defaults to 360°.

    nseg
        Number of areas (8 maximum) around circumference.  Defaults to
        minimum number required for 90° -maximum arcs, i.e., 4 for 360°, 3
        for 270°, etc.

    Notes
    -----
    Generates cylindrical areas (and their corresponding keypoints and
    lines) by rotating a line pattern (and its associated keypoint pattern)
    about an axis.  Keypoint patterns are generated at regular angular
    locations, based on a maximum spacing of 90°.  Line patterns are
    generated at the keypoint patterns.  Arc lines are also generated to
    connect the keypoints circumferentially.  Keypoint, line, and area
    numbers are automatically assigned, beginning with the lowest available
    values [NUMSTR].  Adjacent lines use a common keypoint.  Adjacent areas
    use a common line.
    """
    command = f"AROTAT,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{pax1},{pax2},{arc},{nseg}"
    return self.run(command, **kwargs)


def arscale(self, na1="", na2="", ninc="", rx="", ry="", rz="", kinc="",
            noelem="", imove="", **kwargs):
    """Generates a scaled set of areas from a pattern of areas.

    APDL Command: ARSCALE

    Parameters
    ----------
    na1, na2, ninc
        Set of areas, NA1 to NA2 in steps of NINC, that defines the pattern
        to be scaled.  NA2 defaults to NA1, NINC defaults to 1.  If NA1 =
        ALL, NA2 and NINC are ignored and the pattern is defined by all
        selected areas.  If NA1 = P, graphical picking is enabled and all
        remaining arguments are ignored (valid only in the GUI).  A
        component name may also be substituted for NA1 (NA2 and NINC are
        ignored).

    rx, ry, rz
        Scale factors to be applied to the X, Y, and Z keypoint coordinates
        in the active coordinate system.  (RR, R θ, RZ for cylindrical; RR,
        R θ, R Φ for spherical).  Note that the R θ and R Φ scale factors
        are interpreted as angular offsets.  For example, if CSYS = 1, RX,
        RY, RZ input of (1.5,10,3) would scale the specified keypoints 1.5
        times in the radial and 3 times in the Z direction, while adding an
        offset of 10 degrees to the keypoints.  Zero, blank, or negative
        scale factor values are assumed to be 1.0.  Zero or blank angular
        offsets have no effect.

    kinc
        Increment to be applied to keypoint numbers for generated set.  If
        zero, the lowest available keypoint numbers will be assigned
        [NUMSTR].

    noelem
        Specifies whether nodes and elements are also to be generated:

        0 - Nodes and elements associated with the original areas will be generated
            (scaled) if they exist.

        1 - Nodes and elements will not be generated.

    imove
        Specifies whether areas will be moved or newly defined:

        0 - Additional areas will be generated.

        1 - Original areas will be moved to new position (KINC and NOELEM are ignored).
            Use only if the old areas are no longer needed at their
            original positions.  Corresponding meshed items are also moved
            if not needed at their original position.

    Notes
    -----
    Generates a scaled set of areas (and their corresponding keypoints,
    lines, and mesh) from a pattern of areas.  The MAT, TYPE, REAL, and
    ESYS attributes are based on the areas in the pattern and not the
    current settings.  Scaling is done in the active coordinate system.
    Areas in the pattern could have been generated in any coordinate
    system.  However, solid modeling in a toroidal coordinate system is not
    recommended.
    """
    command = f"ARSCALE,{na1},{na2},{ninc},{rx},{ry},{rz},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def arsym(self, ncomp="", na1="", na2="", ninc="", kinc="", noelem="",
          imove="", **kwargs):
    """Generates areas from an area pattern by symmetry reflection.

    APDL Command: ARSYM

    Parameters
    ----------
    ncomp
        Symmetry key:

        X - X symmetry (default).

        Y - Y symmetry.

        Z - Z symmetry.

    na1, na2, ninc
        Reflect areas from pattern beginning with NA1 to NA2 (defaults to
        NA1) in steps of NINC (defaults to 1).  If NA1 = ALL, NA2 and NINC
        are ignored and the pattern is all selected areas [ASEL].  If Ncomp
        = P, use graphical picking to specify areas and ignore NL2 and
        NINC.  A component name may also be substituted for NA1 (NA2 and
        NINC are ignored).

    kinc
        Keypoint increment between sets.  If zero, the lowest available
        keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies whether nodes and elements are also to be generated:

        0 - Generate nodes and elements associated with the original areas, if they exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether areas will be moved or newly defined:

        0 - Generate additional areas.

        1 - Move original areas to new position retaining the same keypoint numbers (KINC
            and NOELEM are ignored).  Valid only if the old areas are no
            longer needed at their original positions.  Corresponding
            meshed items are also moved if not needed at their original
            position.

    Notes
    -----
    Generates a reflected set of areas (and their corresponding keypoints,
    lines and mesh) from a given area pattern by a symmetry reflection (see
    analogous node symmetry command, NSYM).  The MAT, TYPE, REAL, ESYS, and
    SECNUM attributes are based upon the areas in the pattern and not upon
    the current settings.  Reflection is done in the active coordinate
    system by changing a particular coordinate sign.  The active coordinate
    system must be a Cartesian system.  Areas in the pattern may have been
    generated in any coordinate system.  However, solid modeling in a
    toroidal coordinate system is not recommended.  Areas are generated as
    described in the AGEN command.

    See the ESYM command for additional information about symmetry
    elements.
    """
    command = f"ARSYM,{ncomp},{na1},{na2},{ninc},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def askin(self, nl1="", nl2="", nl3="", nl4="", nl5="", nl6="", nl7="",
          nl8="", nl9="", **kwargs):
    """Generates an area by "skinning" a surface through guiding lines.

    APDL Command: ASKIN

    Parameters
    ----------
    nl1
        The first guiding line forming the skinned area.  If NL1 = P,
        graphical picking is enabled and all remaining arguments are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NL1.  If NL1 is negative, the line beginnings and
        ends will be used to direct the skinning of the remaining lines
        (see "Changing the ASKIN Algorithm" below).

    nl2, nl3, nl4, . . . , nl9
        The additional guiding lines for the skinned area (up to 9 total
        lines, including NL1, if using keyboard entry).  If negative (and
        NL1 is negative), the line beginning and end will be temporarily
        interchanged for the skinning operation (see "Changing the ASKIN
        Algorithm" below).

    Notes
    -----
    Generates an area by "skinning" a surface through specified guiding
    lines.  The lines act as a set of "ribs" over which a surface is
    "stretched."  Two opposite edges of the area are framed by the first
    (NL1) and last (NLn) guiding lines specified.  The other two edges of
    the area are framed by splines-fit lines which the program
    automatically generates through the ends of all guiding lines.  The
    interior of the area is shaped by the interior guiding lines.  Once the
    area has been created, only the four edge lines will be attached to it.
    In rare cases, it may be necessary to change the default algorithm used
    by the ASKIN command (see "Changing the ASKIN Algorithm" below).

    When skinning from one guiding line to the next, the program can create
    the transition area in one of two ways:  one more spiraled and one less
    spiraled ("flatter").  By default, the program attempts to produce the
    flatter transition, instead of the more spiraled transition.  This
    algorithm can be changed by inputting NL1 as a negative number, in
    which case the program connects all the keypoints at the line
    "beginnings" (/PSYMB,LDIR command) as one edge of the area, and all the
    line "ends" as the opposite edge, irrespective of the amount of
    spiraling produced in each transition area.

    To further control the geometry of the area (if NL1 is negative), the
    beginning and end of any specified line (other than NL1) can be
    temporarily interchanged (for the skinning operation only) by inputting
    that line number as negative.  See Solid Modeling in the Modeling and
    Meshing Guide for an illustration.
    """
    command = f"ASKIN,{nl1},{nl2},{nl3},{nl4},{nl5},{nl6},{nl7},{nl8},{nl9}"
    return self.run(command, **kwargs)


def asub(self, na1="", p1="", p2="", p3="", p4="", **kwargs):
    """Generates an area using the shape of an existing area.

    APDL Command: ASUB

    Parameters
    ----------
    na1
        Existing area number whose shape is to be used.  If P1 = P,
        graphical picking is enabled and all remaining arguments are
        ignored (valid only in the GUI).

    p1
        Keypoint defining starting corner of area.

    p2
        Keypoint defining second corner of area.

    p3
        Keypoint defining third corner of area.

    p4
        Keypoint defining fourth corner of area (defaults to P3).

    Notes
    -----
    The new area will overlay the old area.  Often used when the area to be
    subdivided consists of a complex shape that was not generated in a
    single coordinate system.  Keypoints and any corresponding lines must
    lie on the existing area.  Missing lines are generated to lie on the
    given area.  The active coordinate system is ignored.
    """
    command = f"ASUB,{na1},{p1},{p2},{p3},{p4}"
    return self.run(command, **kwargs)


def asum(self, lab="", **kwargs):
    """Calculates and prints geometry statistics of the selected areas.

    APDL Command: ASUM

    Parameters
    ----------
    lab
        Controls the degree of tessellation used in the calculation of area
        properties.  If LAB = DEFAULT, area calculations will use the
        degree of tessellation set through the /FACET command.  If LAB =
        FINE, area calculations are based on a finer tessellation.

    Notes
    -----
    Calculates and prints geometry statistics (area, centroid location,
    moments of inertia, volume, etc.) associated with the selected areas.
    ASUM should only be used on perfectly flat areas.

    Geometry items are reported in the global Cartesian coordinate system.
    A unit thickness is assumed unless the areas have a non-zero total
    thickness defined by real constant or section data.

    For layered areas, a unit density is always assumed. For single-layer
    areas, a unit density is assumed unless the areas have a valid material
    (density).

    The thickness and density are associated to the areas via the AATT
    command.

    Items calculated via ASUM and later retrieved via a *GET or *VGET
    command are valid only if the model is not modified after issuing the
    ASUM command.

    Setting a finer degree of tessellation will provide area calculations
    with greater accuracy, especially for thin, hollow models.  However,
    using a finer degree of tessellation requires longer processing.

    For very narrow (sliver) areas, such that the ratio of the minimum to
    the maximum dimension is less than 0.01, the ASUM command can provide
    erroneous area information.  To ensure that the calculations are
    accurate, subdivide such areas so that the ratio of the minimum to the
    maximum is at least 0.05.
    """
    command = f"ASUM,{lab}"
    return self.run(command, **kwargs)


def atran(self, kcnto="", na1="", na2="", ninc="", kinc="", noelem="",
          imove="", **kwargs):
    """Transfers a pattern of areas to another coordinate system.

    APDL Command: ATRAN

    Parameters
    ----------
    kcnto
        Reference number of coordinate system where the pattern is to be
        transferred.  Transfer occurs from the active coordinate system.
        The coordinate system type and parameters of KCNTO must be the same
        as the active system.

    na1, na2, ninc
        Transfer area pattern beginning with NA1 to NA2 (defaults to NA1)
        in steps of NINC (defaults to 1).  If NA1 = ALL, NA2 and NINC are
        ignored and the pattern is all selected areas [ASEL].  If NA1 = P,
        graphical picking is enabled and all remaining arguments are
        ignored (valid only in the GUI).  A component name may also be
        substituted for NA1 (NA2 and NINC are ignored).

    kinc
        Keypoint increment between sets.  If zero, the lowest available
        keypoint numbers are assigned [NUMSTR].

    noelem
        Specifies whether elements and nodes are also to be generated:

        0 - Generate nodes and elements associated with the original areas, if they exist.

        1 - Do not generate nodes and elements.

    imove
        Specifies whether to redefine the existing areas:

        0 - Generate additional areas.

        1 - Move original areas to new position retaining the same keypoint numbers (KINC
            and NOELEM are ignored).  Valid only if the old areas are no
            longer needed at their original positions.  Corresponding
            meshed items are also moved if not needed at their original
            position.

    Notes
    -----
    Transfers a pattern of areas (and their corresponding lines, keypoints
    and mesh) from one coordinate system to another (see analogous node
    TRANSFER command).  The MAT, TYPE, REAL, and ESYS attributes are based
    upon the areas in the pattern and not upon the current settings.
    Coordinate systems may be translated and rotated relative to each
    other.  Initial pattern may be generated in any coordinate system.
    However, solid modeling in a toroidal coordinate system is not
    recommended.  Coordinate and slope values are interpreted in the active
    coordinate system and are transferred directly.  Areas are generated as
    described in the AGEN command.
    """
    command = f"ATRAN,{kcnto},{na1},{na2},{ninc},{kinc},{noelem},{imove}"
    return self.run(command, **kwargs)


def gsum(self, **kwargs):
    """Calculates and prints geometry items.

    APDL Command: GSUM

    Notes
    -----
    Calculates and prints geometry items (centroid location, moments of
    inertia, length, area, volume etc.) associated with the selected
    keypoints, lines, areas, and volumes. Geometry items are reported in
    the global Cartesian coordinate system.   For volumes, a unit density
    is assumed unless the volumes have a material association via the VATT
    command.  For areas, a unit density (and thickness) is assumed unless
    the areas have a material (and real constant) association via the AATT
    command.  For lines and keypoints, a unit density is assumed,
    irrespective of any material associations [LATT, KATT, MAT].  Items
    calculated by GSUM and later retrieved by a *GET or *VGET commands are
    valid only if the model is not modified after the GSUM command is
    issued.  This command combines the functions of the KSUM, LSUM, ASUM,
    and VSUM commands.
    """
    command = f"GSUM,"
    return self.run(command, **kwargs)


def splot(self, na1="", na2="", ninc="", mesh="", **kwargs):
    """Displays the selected areas and a faceted view of their underlying

    APDL Command: SPLOT
    surfaces

    Parameters
    ----------
    na1
        Starting area for display of areas and underlying surfaces. If NA1
        = ALL (default), NA2 and NINC are ignored and all selected areas
        are displayed (ASEL command).

    na2
        Last area to be displayed.

    ninc
        Numeric value setting steps between NA1 and NA2 for display.
        Default value is (1).

    mesh
        Specifies a rectangular mesh density used to display the underlying
        surface (default 4, i.e. 4 x 4).

    Notes
    -----
    This command is valid in any processor. The plot output displays the
    external and internal trim curves and underlying surface. You cannot
    obtain a faceted view of your surface areas when you are using the
    /EXPAND command to create larger graphics displays.

    Use APLOT for trimmed surface display.
    """
    command = f"SPLOT,{na1},{na2},{ninc},{mesh}"
    return self.run(command, **kwargs)
